@@ -34,14 +34,13 @@ struct multimedia_stream
 {
     IAMMultiMediaStream IAMMultiMediaStream_iface;
     LONG ref;
-    IGraphBuilder* pFilterGraph;
+    IGraphBuilder *graph;
     IMediaSeeking* media_seeking;
     IMediaControl* media_control;
     IMediaStreamFilter *filter;
     IPin* ipin;
-    ULONG nbStreams;
-    IAMMediaStream **pStreams;
-    STREAM_TYPE StreamType;
+    BOOL initialized;
+    STREAM_TYPE type;
     OAEVENT event;
 };
 
@@ -50,39 +49,9 @@ static inline struct multimedia_stream *impl_from_IAMMultiMediaStream(IAMMultiMe
     return CONTAINING_RECORD(iface, struct multimedia_stream, IAMMultiMediaStream_iface);
 }
 
-static const struct IAMMultiMediaStreamVtbl AM_Vtbl;
-
-HRESULT AM_create(IUnknown *pUnkOuter, LPVOID *ppObj)
-{
-    struct multimedia_stream *object;
-    HRESULT hr;
-
-    TRACE("(%p,%p)\n", pUnkOuter, ppObj);
-
-    if( pUnkOuter )
-        return CLASS_E_NOAGGREGATION;
-
-    if (!(object = heap_alloc_zero(sizeof(*object))))
-        return E_OUTOFMEMORY;
-
-    object->IAMMultiMediaStream_iface.lpVtbl = &AM_Vtbl;
-    object->ref = 1;
-
-    if (FAILED(hr = CoCreateInstance(&CLSID_MediaStreamFilter, NULL,
-            CLSCTX_INPROC_SERVER, &IID_IMediaStreamFilter, (void **)&object->filter)))
-    {
-        ERR("Failed to create stream filter, hr %#x.\n", hr);
-        heap_free(object);
-        return hr;
-    }
-
-    *ppObj = &object->IAMMultiMediaStream_iface;
-
-    return S_OK;
-}
-
 /*** IUnknown methods ***/
-static HRESULT WINAPI IAMMultiMediaStreamImpl_QueryInterface(IAMMultiMediaStream* iface, REFIID riid, void** ppvObject)
+static HRESULT WINAPI multimedia_stream_QueryInterface(IAMMultiMediaStream *iface,
+        REFIID riid, void **ppvObject)
 {
     struct multimedia_stream *This = impl_from_IAMMultiMediaStream(iface);
 
@@ -102,7 +71,7 @@ static HRESULT WINAPI IAMMultiMediaStreamImpl_QueryInterface(IAMMultiMediaStream
     return E_NOINTERFACE;
 }
 
-static ULONG WINAPI IAMMultiMediaStreamImpl_AddRef(IAMMultiMediaStream* iface)
+static ULONG WINAPI multimedia_stream_AddRef(IAMMultiMediaStream *iface)
 {
     struct multimedia_stream *This = impl_from_IAMMultiMediaStream(iface);
 
@@ -111,19 +80,15 @@ static ULONG WINAPI IAMMultiMediaStreamImpl_AddRef(IAMMultiMediaStream* iface)
     return InterlockedIncrement(&This->ref);
 }
 
-static ULONG WINAPI IAMMultiMediaStreamImpl_Release(IAMMultiMediaStream* iface)
+static ULONG WINAPI multimedia_stream_Release(IAMMultiMediaStream *iface)
 {
     struct multimedia_stream *This = impl_from_IAMMultiMediaStream(iface);
     ULONG ref = InterlockedDecrement(&This->ref);
-    ULONG i;
 
     TRACE("(%p/%p)\n", iface, This);
 
     if (!ref)
     {
-        for(i = 0; i < This->nbStreams; i++)
-            IAMMediaStream_Release(This->pStreams[i]);
-        CoTaskMemFree(This->pStreams);
         if (This->ipin)
             IPin_Release(This->ipin);
         IMediaStreamFilter_Release(This->filter);
@@ -131,8 +96,8 @@ static ULONG WINAPI IAMMultiMediaStreamImpl_Release(IAMMultiMediaStream* iface)
             IMediaSeeking_Release(This->media_seeking);
         if (This->media_control)
             IMediaControl_Release(This->media_control);
-        if (This->pFilterGraph)
-            IGraphBuilder_Release(This->pFilterGraph);
+        if (This->graph)
+            IGraphBuilder_Release(This->graph);
         HeapFree(GetProcessHeap(), 0, This);
     }
 
@@ -140,7 +105,8 @@ static ULONG WINAPI IAMMultiMediaStreamImpl_Release(IAMMultiMediaStream* iface)
 }
 
 /*** IMultiMediaStream methods ***/
-static HRESULT WINAPI IAMMultiMediaStreamImpl_GetInformation(IAMMultiMediaStream* iface, DWORD* pdwFlags, STREAM_TYPE* pStreamType)
+static HRESULT WINAPI multimedia_stream_GetInformation(IAMMultiMediaStream *iface,
+        DWORD *pdwFlags, STREAM_TYPE *pStreamType)
 {
     struct multimedia_stream *This = impl_from_IAMMultiMediaStream(iface);
 
@@ -149,38 +115,27 @@ static HRESULT WINAPI IAMMultiMediaStreamImpl_GetInformation(IAMMultiMediaStream
     return E_NOTIMPL;
 }
 
-static HRESULT WINAPI IAMMultiMediaStreamImpl_GetMediaStream(IAMMultiMediaStream* iface, REFMSPID idPurpose, IMediaStream** ppMediaStream)
+static HRESULT WINAPI multimedia_stream_GetMediaStream(IAMMultiMediaStream *iface,
+        REFMSPID id, IMediaStream **stream)
 {
-    struct multimedia_stream *This = impl_from_IAMMultiMediaStream(iface);
-    MSPID PurposeId;
-    unsigned int i;
+    struct multimedia_stream *mmstream = impl_from_IAMMultiMediaStream(iface);
 
-    TRACE("(%p/%p)->(%s,%p)\n", This, iface, debugstr_guid(idPurpose), ppMediaStream);
+    TRACE("mmstream %p, id %s, stream %p.\n", mmstream, debugstr_guid(id), stream);
 
-    for (i = 0; i < This->nbStreams; i++)
-    {
-        IAMMediaStream_GetInformation(This->pStreams[i], &PurposeId, NULL);
-        if (IsEqualIID(&PurposeId, idPurpose))
-        {
-            *ppMediaStream = (IMediaStream*)This->pStreams[i];
-            IMediaStream_AddRef(*ppMediaStream);
-            return S_OK;
-        }
-    }
-
-    return MS_E_NOSTREAM;
+    return IMediaStreamFilter_GetMediaStream(mmstream->filter, id, stream);
 }
 
-static HRESULT WINAPI IAMMultiMediaStreamImpl_EnumMediaStreams(IAMMultiMediaStream* iface, LONG Index, IMediaStream** ppMediaStream)
+static HRESULT WINAPI multimedia_stream_EnumMediaStreams(IAMMultiMediaStream *iface,
+        LONG index, IMediaStream **stream)
 {
-    struct multimedia_stream *This = impl_from_IAMMultiMediaStream(iface);
+    struct multimedia_stream *mmstream = impl_from_IAMMultiMediaStream(iface);
 
-    FIXME("(%p/%p)->(%d,%p) stub!\n", This, iface, Index, ppMediaStream);
+    TRACE("mmstream %p, index %d, stream %p.\n", mmstream, index, stream);
 
-    return E_NOTIMPL;
+    return IMediaStreamFilter_EnumMediaStreams(mmstream->filter, index, stream);
 }
 
-static HRESULT WINAPI IAMMultiMediaStreamImpl_GetState(IAMMultiMediaStream* iface, STREAM_STATE* pCurrentState)
+static HRESULT WINAPI multimedia_stream_GetState(IAMMultiMediaStream *iface, STREAM_STATE *pCurrentState)
 {
     struct multimedia_stream *This = impl_from_IAMMultiMediaStream(iface);
 
@@ -189,7 +144,7 @@ static HRESULT WINAPI IAMMultiMediaStreamImpl_GetState(IAMMultiMediaStream* ifac
     return E_NOTIMPL;
 }
 
-static HRESULT WINAPI IAMMultiMediaStreamImpl_SetState(IAMMultiMediaStream* iface, STREAM_STATE new_state)
+static HRESULT WINAPI multimedia_stream_SetState(IAMMultiMediaStream *iface, STREAM_STATE new_state)
 {
     struct multimedia_stream *This = impl_from_IAMMultiMediaStream(iface);
     HRESULT hr = E_INVALIDARG;
@@ -204,7 +159,7 @@ static HRESULT WINAPI IAMMultiMediaStreamImpl_SetState(IAMMultiMediaStream* ifac
     return hr;
 }
 
-static HRESULT WINAPI IAMMultiMediaStreamImpl_GetTime(IAMMultiMediaStream* iface, STREAM_TIME* pCurrentTime)
+static HRESULT WINAPI multimedia_stream_GetTime(IAMMultiMediaStream *iface, STREAM_TIME *pCurrentTime)
 {
     struct multimedia_stream *This = impl_from_IAMMultiMediaStream(iface);
 
@@ -213,7 +168,7 @@ static HRESULT WINAPI IAMMultiMediaStreamImpl_GetTime(IAMMultiMediaStream* iface
     return E_NOTIMPL;
 }
 
-static HRESULT WINAPI IAMMultiMediaStreamImpl_GetDuration(IAMMultiMediaStream* iface, STREAM_TIME* pDuration)
+static HRESULT WINAPI multimedia_stream_GetDuration(IAMMultiMediaStream *iface, STREAM_TIME *pDuration)
 {
     struct multimedia_stream *This = impl_from_IAMMultiMediaStream(iface);
 
@@ -222,7 +177,7 @@ static HRESULT WINAPI IAMMultiMediaStreamImpl_GetDuration(IAMMultiMediaStream* i
     return E_NOTIMPL;
 }
 
-static HRESULT WINAPI IAMMultiMediaStreamImpl_Seek(IAMMultiMediaStream* iface, STREAM_TIME seek_time)
+static HRESULT WINAPI multimedia_stream_Seek(IAMMultiMediaStream *iface, STREAM_TIME seek_time)
 {
     struct multimedia_stream *This = impl_from_IAMMultiMediaStream(iface);
 
@@ -231,7 +186,7 @@ static HRESULT WINAPI IAMMultiMediaStreamImpl_Seek(IAMMultiMediaStream* iface, S
     return IMediaSeeking_SetPositions(This->media_seeking, &seek_time, AM_SEEKING_AbsolutePositioning, NULL, AM_SEEKING_NoPositioning);
 }
 
-static HRESULT WINAPI IAMMultiMediaStreamImpl_GetEndOfStream(IAMMultiMediaStream* iface, HANDLE* phEOS)
+static HRESULT WINAPI multimedia_stream_GetEndOfStream(IAMMultiMediaStream *iface, HANDLE *phEOS)
 {
     struct multimedia_stream *This = impl_from_IAMMultiMediaStream(iface);
 
@@ -240,80 +195,96 @@ static HRESULT WINAPI IAMMultiMediaStreamImpl_GetEndOfStream(IAMMultiMediaStream
     return E_NOTIMPL;
 }
 
-/*** IAMMultiMediaStream methods ***/
-static HRESULT WINAPI IAMMultiMediaStreamImpl_Initialize(IAMMultiMediaStream* iface, STREAM_TYPE StreamType, DWORD dwFlags, IGraphBuilder* pFilterGraph)
+static HRESULT create_graph(struct multimedia_stream *mmstream, IGraphBuilder *graph)
 {
-    static const WCHAR filternameW[] = {'M','e','d','i','a','S','t','r','e','a','m','F','i','l','t','e','r',0};
-    struct multimedia_stream *This = impl_from_IAMMultiMediaStream(iface);
-    HRESULT hr = S_OK;
+    IMediaEventEx *eventsrc;
+    HRESULT hr;
 
-    TRACE("(%p/%p)->(%x,%x,%p)\n", This, iface, (DWORD)StreamType, dwFlags, pFilterGraph);
-
-    if (pFilterGraph)
-    {
-        This->pFilterGraph = pFilterGraph;
-        IGraphBuilder_AddRef(This->pFilterGraph);
-    }
-    else
-    {
-        hr = CoCreateInstance(&CLSID_FilterGraph, NULL, CLSCTX_INPROC_SERVER, &IID_IGraphBuilder, (LPVOID*)&This->pFilterGraph);
-    }
+    if (graph)
+        IGraphBuilder_AddRef(mmstream->graph = graph);
+    else if (FAILED(hr = CoCreateInstance(&CLSID_FilterGraph, NULL,
+            CLSCTX_INPROC_SERVER, &IID_IGraphBuilder, (void **)&mmstream->graph)))
+        return hr;
 
+    hr = IGraphBuilder_QueryInterface(mmstream->graph, &IID_IMediaSeeking, (void **)&mmstream->media_seeking);
+    if (SUCCEEDED(hr))
+        hr = IGraphBuilder_QueryInterface(mmstream->graph, &IID_IMediaControl, (void **)&mmstream->media_control);
+    if (SUCCEEDED(hr))
+        hr = IGraphBuilder_AddFilter(mmstream->graph, (IBaseFilter *)mmstream->filter, L"MediaStreamFilter");
+    if (SUCCEEDED(hr))
+        hr = IGraphBuilder_QueryInterface(mmstream->graph, &IID_IMediaEventEx, (void **)&eventsrc);
     if (SUCCEEDED(hr))
     {
-        This->StreamType = StreamType;
-        hr = IGraphBuilder_QueryInterface(This->pFilterGraph, &IID_IMediaSeeking, (void**)&This->media_seeking);
+        hr = IMediaEventEx_GetEventHandle(eventsrc, &mmstream->event);
         if (SUCCEEDED(hr))
-            hr = IGraphBuilder_QueryInterface(This->pFilterGraph, &IID_IMediaControl, (void**)&This->media_control);
-        if (SUCCEEDED(hr))
-            hr = IGraphBuilder_AddFilter(This->pFilterGraph, (IBaseFilter*)This->filter, filternameW);
-        if (SUCCEEDED(hr))
-        {
-            IMediaEventEx* media_event = NULL;
-            hr = IGraphBuilder_QueryInterface(This->pFilterGraph, &IID_IMediaEventEx, (void**)&media_event);
-            if (SUCCEEDED(hr))
-                hr = IMediaEventEx_GetEventHandle(media_event, &This->event);
-            if (SUCCEEDED(hr))
-                hr = IMediaEventEx_SetNotifyFlags(media_event, AM_MEDIAEVENT_NONOTIFY);
-            if (media_event)
-                IMediaEventEx_Release(media_event);
-        }
+            hr = IMediaEventEx_SetNotifyFlags(eventsrc, AM_MEDIAEVENT_NONOTIFY);
+        IMediaEventEx_Release(eventsrc);
     }
 
     if (FAILED(hr))
     {
-        if (This->media_seeking)
-            IMediaSeeking_Release(This->media_seeking);
-        This->media_seeking = NULL;
-        if (This->media_control)
-            IMediaControl_Release(This->media_control);
-        This->media_control = NULL;
-        if (This->pFilterGraph)
-            IGraphBuilder_Release(This->pFilterGraph);
-        This->pFilterGraph = NULL;
+        if (mmstream->media_seeking)
+            IMediaSeeking_Release(mmstream->media_seeking);
+        mmstream->media_seeking = NULL;
+        if (mmstream->media_control)
+            IMediaControl_Release(mmstream->media_control);
+        mmstream->media_control = NULL;
+        if (mmstream->graph)
+            IGraphBuilder_Release(mmstream->graph);
+        mmstream->graph = NULL;
     }
 
     return hr;
 }
 
-static HRESULT WINAPI IAMMultiMediaStreamImpl_GetFilterGraph(IAMMultiMediaStream* iface, IGraphBuilder** ppGraphBuilder)
+static HRESULT WINAPI multimedia_stream_Initialize(IAMMultiMediaStream *iface,
+        STREAM_TYPE type, DWORD flags, IGraphBuilder *graph)
 {
-    struct multimedia_stream *This = impl_from_IAMMultiMediaStream(iface);
+    struct multimedia_stream *mmstream = impl_from_IAMMultiMediaStream(iface);
+    HRESULT hr;
 
-    TRACE("(%p/%p)->(%p)\n", This, iface, ppGraphBuilder);
+    TRACE("mmstream %p, type %u, flags %#x, graph %p.\n", mmstream, type, flags, graph);
 
-    if (!ppGraphBuilder)
-        return E_POINTER;
+    if (graph && mmstream->graph)
+    {
+        WARN("Graph already initialized, returning E_INVALIDARG.\n");
+        return E_INVALIDARG;
+    }
 
-    if (This->pFilterGraph)
-        return IGraphBuilder_QueryInterface(This->pFilterGraph, &IID_IGraphBuilder, (void**)ppGraphBuilder);
-    else
-        *ppGraphBuilder = NULL;
+    if (mmstream->initialized && type != mmstream->type)
+    {
+        WARN("Attempt to change type from %u, returning E_INVALIDARG.\n", mmstream->type);
+        return E_INVALIDARG;
+    }
+
+    if (graph && FAILED(hr = create_graph(mmstream, graph)))
+        return hr;
+
+    mmstream->type = type;
+    mmstream->initialized = TRUE;
 
     return S_OK;
 }
 
-static HRESULT WINAPI IAMMultiMediaStreamImpl_GetFilter(IAMMultiMediaStream *iface, IMediaStreamFilter **filter)
+static HRESULT WINAPI multimedia_stream_GetFilterGraph(IAMMultiMediaStream *iface, IGraphBuilder **graph)
+{
+    struct multimedia_stream *mmstream = impl_from_IAMMultiMediaStream(iface);
+
+    TRACE("mmstream %p, graph %p.\n", mmstream, graph);
+
+    if (!graph)
+        return E_POINTER;
+
+    if (mmstream->graph)
+        IGraphBuilder_AddRef(*graph = mmstream->graph);
+    else
+        *graph = NULL;
+
+    return S_OK;
+}
+
+static HRESULT WINAPI multimedia_stream_GetFilter(IAMMultiMediaStream *iface,
+        IMediaStreamFilter **filter)
 {
     struct multimedia_stream *mmstream = impl_from_IAMMultiMediaStream(iface);
 
@@ -327,77 +298,72 @@ static HRESULT WINAPI IAMMultiMediaStreamImpl_GetFilter(IAMMultiMediaStream *ifa
     return S_OK;
 }
 
-static HRESULT WINAPI IAMMultiMediaStreamImpl_AddMediaStream(IAMMultiMediaStream* iface, IUnknown* stream_object, const MSPID* PurposeId,
-                                          DWORD dwFlags, IMediaStream** ppNewStream)
+static HRESULT WINAPI multimedia_stream_AddMediaStream(IAMMultiMediaStream *iface,
+        IUnknown *stream_object, const MSPID *PurposeId, DWORD dwFlags, IMediaStream **ret_stream)
 {
     struct multimedia_stream *This = impl_from_IAMMultiMediaStream(iface);
     HRESULT hr;
     IAMMediaStream* pStream;
-    IAMMediaStream** pNewStreams;
+    IMediaStream *stream;
 
-    TRACE("(%p/%p)->(%p,%s,%x,%p)\n", This, iface, stream_object, debugstr_guid(PurposeId), dwFlags, ppNewStream);
+    TRACE("mmstream %p, stream_object %p, id %s, flags %#x, ret_stream %p.\n",
+            This, stream_object, debugstr_guid(PurposeId), dwFlags, ret_stream);
 
     if (!IsEqualGUID(PurposeId, &MSPID_PrimaryVideo) && !IsEqualGUID(PurposeId, &MSPID_PrimaryAudio))
         return MS_E_PURPOSEID;
 
+    if (IMediaStreamFilter_GetMediaStream(This->filter, PurposeId, &stream) == S_OK)
+    {
+        IMediaStream_Release(stream);
+        return MS_E_PURPOSEID;
+    }
+
+    if (!This->graph && FAILED(hr = create_graph(This, NULL)))
+        return hr;
+
     if (dwFlags & AMMSF_ADDDEFAULTRENDERER)
     {
-        if (IsEqualGUID(PurposeId, &MSPID_PrimaryVideo))
+        IBaseFilter *dsound_render;
+
+        if (ret_stream)
+            return E_INVALIDARG;
+
+        if (!IsEqualGUID(PurposeId, &MSPID_PrimaryAudio))
         {
-            /* Default renderer not supported by video stream */
+            WARN("AMMSF_ADDDEFAULTRENDERER requested with id %s, returning MS_E_PURPOSEID.\n", debugstr_guid(PurposeId));
             return MS_E_PURPOSEID;
         }
-        else
+
+        if (SUCCEEDED(hr = CoCreateInstance(&CLSID_DSoundRender, NULL,
+                CLSCTX_INPROC_SERVER, &IID_IBaseFilter, (void **)&dsound_render)))
         {
-            IBaseFilter* dsoundrender_filter;
-
-            /* Create the default renderer for audio */
-            hr = CoCreateInstance(&CLSID_DSoundRender, NULL, CLSCTX_INPROC_SERVER, &IID_IBaseFilter, (LPVOID*)&dsoundrender_filter);
-            if (SUCCEEDED(hr))
-            {
-                 hr = IGraphBuilder_AddFilter(This->pFilterGraph, dsoundrender_filter, NULL);
-                 IBaseFilter_Release(dsoundrender_filter);
-            }
-
-            /* No media stream created when the default renderer is used */
-            return hr;
+            hr = IGraphBuilder_AddFilter(This->graph, dsound_render, NULL);
+            IBaseFilter_Release(dsound_render);
         }
+        return hr;
     }
 
     if (IsEqualGUID(PurposeId, &MSPID_PrimaryVideo))
-        hr = ddrawmediastream_create((IMultiMediaStream*)iface, PurposeId, stream_object, This->StreamType, &pStream);
+        hr = ddraw_stream_create((IMultiMediaStream*)iface, PurposeId, stream_object, This->type, &pStream);
     else
-        hr = audiomediastream_create((IMultiMediaStream*)iface, PurposeId, stream_object, This->StreamType, &pStream);
-    if (SUCCEEDED(hr))
-    {
-        pNewStreams = CoTaskMemRealloc(This->pStreams, (This->nbStreams+1) * sizeof(IAMMediaStream*));
-        if (!pNewStreams)
-        {
-            IAMMediaStream_Release(pStream);
-            return E_OUTOFMEMORY;
-        }
-        This->pStreams = pNewStreams;
-        This->pStreams[This->nbStreams] = pStream;
-        This->nbStreams++;
-
-        if (ppNewStream)
-        {
-            IMediaStream_AddRef(*ppNewStream = (IMediaStream*)pStream);
-        }
-    }
+        hr = audio_stream_create((IMultiMediaStream*)iface, PurposeId, stream_object, This->type, &pStream);
 
     if (SUCCEEDED(hr))
     {
         /* Add stream to the media stream filter */
         IMediaStreamFilter_AddMediaStream(This->filter, pStream);
+        if (ret_stream)
+            *ret_stream = (IMediaStream *)pStream;
+        else
+            IAMMediaStream_Release(pStream);
     }
 
     return hr;
 }
 
-static HRESULT WINAPI IAMMultiMediaStreamImpl_OpenFile(IAMMultiMediaStream* iface, LPCWSTR filename, DWORD flags)
+static HRESULT WINAPI multimedia_stream_OpenFile(IAMMultiMediaStream *iface,
+        const WCHAR *filename, DWORD flags)
 {
-    static const WCHAR sourceW[] = {'S','o','u','r','c','e',0};
     struct multimedia_stream *This = impl_from_IAMMultiMediaStream(iface);
     HRESULT ret = S_OK;
     IBaseFilter *BaseFilter = NULL;
@@ -411,11 +377,15 @@ static HRESULT WINAPI IAMMultiMediaStreamImpl_OpenFile(IAMMultiMediaStream* ifac
         return E_POINTER;
 
     /* If Initialize was not called before, we do it here */
-    if (!This->pFilterGraph)
+    if (!This->graph)
+    {
         ret = IAMMultiMediaStream_Initialize(iface, STREAMTYPE_READ, 0, NULL);
+        if (SUCCEEDED(ret))
+            ret = create_graph(This, NULL);
+    }
 
     if (SUCCEEDED(ret))
-        ret = IGraphBuilder_AddSourceFilter(This->pFilterGraph, filename, sourceW, &BaseFilter);
+        ret = IGraphBuilder_AddSourceFilter(This->graph, filename, L"Source", &BaseFilter);
 
     if (SUCCEEDED(ret))
         ret = IBaseFilter_EnumPins(BaseFilter, &EnumPins);
@@ -431,7 +401,7 @@ static HRESULT WINAPI IAMMultiMediaStreamImpl_OpenFile(IAMMultiMediaStream* ifac
     }
 
     if (SUCCEEDED(ret) && !(flags & AMMSF_NORENDER))
-        ret = IGraphBuilder_Render(This->pFilterGraph, This->ipin);
+        ret = IGraphBuilder_Render(This->graph, This->ipin);
 
     if (EnumPins)
         IEnumPins_Release(EnumPins);
@@ -440,7 +410,8 @@ static HRESULT WINAPI IAMMultiMediaStreamImpl_OpenFile(IAMMultiMediaStream* ifac
     return ret;
 }
 
-static HRESULT WINAPI IAMMultiMediaStreamImpl_OpenMoniker(IAMMultiMediaStream* iface, IBindCtx* pCtx, IMoniker* pMoniker, DWORD dwFlags)
+static HRESULT WINAPI multimedia_stream_OpenMoniker(IAMMultiMediaStream *iface,
+        IBindCtx *pCtx, IMoniker *pMoniker, DWORD dwFlags)
 {
     struct multimedia_stream *This = impl_from_IAMMultiMediaStream(iface);
 
@@ -449,7 +420,7 @@ static HRESULT WINAPI IAMMultiMediaStreamImpl_OpenMoniker(IAMMultiMediaStream* i
     return E_NOTIMPL;
 }
 
-static HRESULT WINAPI IAMMultiMediaStreamImpl_Render(IAMMultiMediaStream* iface, DWORD dwFlags)
+static HRESULT WINAPI multimedia_stream_Render(IAMMultiMediaStream *iface, DWORD dwFlags)
 {
     struct multimedia_stream *This = impl_from_IAMMultiMediaStream(iface);
 
@@ -458,28 +429,56 @@ static HRESULT WINAPI IAMMultiMediaStreamImpl_Render(IAMMultiMediaStream* iface,
     if(dwFlags != AMMSF_NOCLOCK)
         return E_INVALIDARG;
 
-    return IGraphBuilder_Render(This->pFilterGraph, This->ipin);
+    return IGraphBuilder_Render(This->graph, This->ipin);
 }
 
-static const IAMMultiMediaStreamVtbl AM_Vtbl =
+static const IAMMultiMediaStreamVtbl multimedia_stream_vtbl =
 {
-    IAMMultiMediaStreamImpl_QueryInterface,
-    IAMMultiMediaStreamImpl_AddRef,
-    IAMMultiMediaStreamImpl_Release,
-    IAMMultiMediaStreamImpl_GetInformation,
-    IAMMultiMediaStreamImpl_GetMediaStream,
-    IAMMultiMediaStreamImpl_EnumMediaStreams,
-    IAMMultiMediaStreamImpl_GetState,
-    IAMMultiMediaStreamImpl_SetState,
-    IAMMultiMediaStreamImpl_GetTime,
-    IAMMultiMediaStreamImpl_GetDuration,
-    IAMMultiMediaStreamImpl_Seek,
-    IAMMultiMediaStreamImpl_GetEndOfStream,
-    IAMMultiMediaStreamImpl_Initialize,
-    IAMMultiMediaStreamImpl_GetFilterGraph,
-    IAMMultiMediaStreamImpl_GetFilter,
-    IAMMultiMediaStreamImpl_AddMediaStream,
-    IAMMultiMediaStreamImpl_OpenFile,
-    IAMMultiMediaStreamImpl_OpenMoniker,
-    IAMMultiMediaStreamImpl_Render
+    multimedia_stream_QueryInterface,
+    multimedia_stream_AddRef,
+    multimedia_stream_Release,
+    multimedia_stream_GetInformation,
+    multimedia_stream_GetMediaStream,
+    multimedia_stream_EnumMediaStreams,
+    multimedia_stream_GetState,
+    multimedia_stream_SetState,
+    multimedia_stream_GetTime,
+    multimedia_stream_GetDuration,
+    multimedia_stream_Seek,
+    multimedia_stream_GetEndOfStream,
+    multimedia_stream_Initialize,
+    multimedia_stream_GetFilterGraph,
+    multimedia_stream_GetFilter,
+    multimedia_stream_AddMediaStream,
+    multimedia_stream_OpenFile,
+    multimedia_stream_OpenMoniker,
+    multimedia_stream_Render
 };
+
+HRESULT multimedia_stream_create(IUnknown *outer, void **out)
+{
+    struct multimedia_stream *object;
+    HRESULT hr;
+
+    if (outer)
+        return CLASS_E_NOAGGREGATION;
+
+    if (!(object = heap_alloc_zero(sizeof(*object))))
+        return E_OUTOFMEMORY;
+
+    object->IAMMultiMediaStream_iface.lpVtbl = &multimedia_stream_vtbl;
+    object->ref = 1;
+
+    if (FAILED(hr = CoCreateInstance(&CLSID_MediaStreamFilter, NULL,
+            CLSCTX_INPROC_SERVER, &IID_IMediaStreamFilter, (void **)&object->filter)))
+    {
+        ERR("Failed to create stream filter, hr %#x.\n", hr);
+        heap_free(object);
+        return hr;
+    }
+
+    TRACE("Created multimedia stream %p.\n", object);
+    *out = &object->IAMMultiMediaStream_iface;
+
+    return S_OK;
+}
