@@ -35,7 +35,6 @@
 #include "winuser.h"
 #include "wine/debug.h"
 #include "wine/heap.h"
-#include "wine/unicode.h"
 
 #include "d3d9.h"
 #include "wine/wined3d.h"
@@ -48,6 +47,9 @@
 
 #define D3D9_TEXTURE_MIPMAP_DIRTY 0x1
 
+#define D3DFMT_RESZ MAKEFOURCC('R','E','S','Z')
+#define D3D9_RESZ_CODE 0x7fa05000
+
 extern const struct wined3d_parent_ops d3d9_null_wined3d_parent_ops DECLSPEC_HIDDEN;
 
 HRESULT vdecl_convert_fvf(DWORD FVF, D3DVERTEXELEMENT9 **ppVertexElements) DECLSPEC_HIDDEN;
@@ -64,6 +66,8 @@ struct d3d9
     IDirect3D9Ex IDirect3D9Ex_iface;
     LONG refcount;
     struct wined3d *wined3d;
+    struct wined3d_output **wined3d_outputs;
+    unsigned int wined3d_output_count;
     BOOL extended;
 };
 
@@ -88,6 +92,7 @@ struct d3d9_device
     struct wined3d_device_parent device_parent;
     LONG refcount;
     struct wined3d_device *wined3d_device;
+    unsigned int adapter_ordinal;
     struct d3d9 *d3d_parent;
 
     struct fvf_declaration *fvf_decls;
@@ -108,15 +113,17 @@ struct d3d9_device
     DWORD in_destruction : 1;
     DWORD in_scene : 1;
     DWORD has_vertex_declaration : 1;
-    DWORD recording : 1;
-    DWORD padding : 11;
+    DWORD padding : 12;
 
     DWORD auto_mipmaps; /* D3D9_MAX_TEXTURE_UNITS */
 
-    unsigned int max_user_clip_planes;
+    unsigned int max_user_clip_planes, vs_uniform_count;
 
     UINT implicit_swapchain_count;
-    struct d3d9_swapchain **implicit_swapchains;
+    struct wined3d_swapchain **implicit_swapchains;
+
+    struct wined3d_stateblock *recording, *state, *update_state;
+    const struct wined3d_stateblock_state *stateblock_state;
 };
 
 HRESULT device_init(struct d3d9_device *device, struct d3d9 *parent, struct wined3d *wined3d,
