@@ -291,10 +291,10 @@ static const char *debug_print_swizzle(DWORD arg)
             return ".w";
     }
 
-    swizzle[0] = (arg >> (BWRITERVS_SWIZZLE_SHIFT + 0)) & 0x03;
-    swizzle[1] = (arg >> (BWRITERVS_SWIZZLE_SHIFT + 2)) & 0x03;
-    swizzle[2] = (arg >> (BWRITERVS_SWIZZLE_SHIFT + 4)) & 0x03;
-    swizzle[3] = (arg >> (BWRITERVS_SWIZZLE_SHIFT + 6)) & 0x03;
+    swizzle[0] = arg & 3;
+    swizzle[1] = (arg >> 2) & 3;
+    swizzle[2] = (arg >> 4) & 3;
+    swizzle[3] = (arg >> 6) & 3;
 
     ret[0] = '.';
     for (i = 0; i < 4; ++i)
@@ -758,6 +758,7 @@ void compilation_message(struct compilation_messages *msg, const char *fmt, __ms
     }
 }
 
+#if D3D_COMPILER_VERSION
 BOOL add_declaration(struct hlsl_scope *scope, struct hlsl_ir_var *decl, BOOL local_var)
 {
     struct hlsl_ir_var *var;
@@ -1190,8 +1191,7 @@ static struct hlsl_type *expr_common_type(struct hlsl_type *t1, struct hlsl_type
 
     if (t1->type > HLSL_CLASS_LAST_NUMERIC || t2->type > HLSL_CLASS_LAST_NUMERIC)
     {
-        hlsl_report_message(loc->file, loc->line, loc->col, HLSL_LEVEL_ERROR,
-                "non scalar/vector/matrix data type in expression");
+        hlsl_report_message(*loc, HLSL_LEVEL_ERROR, "non scalar/vector/matrix data type in expression");
         return NULL;
     }
 
@@ -1200,8 +1200,7 @@ static struct hlsl_type *expr_common_type(struct hlsl_type *t1, struct hlsl_type
 
     if (!expr_compatible_data_types(t1, t2))
     {
-        hlsl_report_message(loc->file, loc->line, loc->col, HLSL_LEVEL_ERROR,
-                "expression data types are incompatible");
+        hlsl_report_message(*loc, HLSL_LEVEL_ERROR, "expression data types are incompatible");
         return NULL;
     }
 
@@ -1274,13 +1273,31 @@ static struct hlsl_type *expr_common_type(struct hlsl_type *t1, struct hlsl_type
     return new_hlsl_type(NULL, type, base, dimx, dimy);
 }
 
-static struct hlsl_ir_node *implicit_conversion(struct hlsl_ir_node *node, struct hlsl_type *type,
+struct hlsl_ir_node *implicit_conversion(struct hlsl_ir_node *node, struct hlsl_type *dst_type,
         struct source_location *loc)
 {
-    if (compare_hlsl_types(node->data_type, type))
+    struct hlsl_type *src_type = node->data_type;
+    struct hlsl_ir_expr *cast;
+
+    if (compare_hlsl_types(src_type, dst_type))
         return node;
-    TRACE("Implicit conversion of expression to %s\n", debug_hlsl_type(type));
-    return &new_cast(node, type, loc)->node;
+
+    if (!implicit_compatible_data_types(src_type, dst_type))
+    {
+        hlsl_report_message(*loc, HLSL_LEVEL_ERROR, "can't implicitly convert %s to %s",
+                debug_hlsl_type(src_type), debug_hlsl_type(dst_type));
+        return NULL;
+    }
+
+    if (dst_type->dimx * dst_type->dimy < src_type->dimx * src_type->dimy)
+        hlsl_report_message(*loc, HLSL_LEVEL_WARNING, "implicit truncation of vector type");
+
+    TRACE("Implicit conversion from %s to %s.\n", debug_hlsl_type(src_type), debug_hlsl_type(dst_type));
+
+    if (!(cast = new_cast(node, dst_type, loc)))
+        return NULL;
+    list_add_after(&node->entry, &cast->node.entry);
+    return &cast->node;
 }
 
 struct hlsl_ir_expr *new_expr(enum hlsl_ir_expr_op op, struct hlsl_ir_node **operands,
@@ -1311,6 +1328,8 @@ struct hlsl_ir_expr *new_expr(enum hlsl_ir_expr_op op, struct hlsl_ir_node **ope
     }
     for (i = 0; i <= 2; ++i)
     {
+        struct hlsl_ir_expr *cast;
+
         if (!operands[i])
             break;
         if (compare_hlsl_types(operands[i]->data_type, type))
@@ -1319,17 +1338,16 @@ struct hlsl_ir_expr *new_expr(enum hlsl_ir_expr_op op, struct hlsl_ir_node **ope
         if (operands[i]->data_type->dimx * operands[i]->data_type->dimy != 1
                 && operands[i]->data_type->dimx * operands[i]->data_type->dimy != type->dimx * type->dimy)
         {
-            hlsl_report_message(operands[i]->loc.file,
-                    operands[i]->loc.line, operands[i]->loc.col, HLSL_LEVEL_WARNING,
-                    "implicit truncation of vector/matrix type");
+            hlsl_report_message(operands[i]->loc, HLSL_LEVEL_WARNING, "implicit truncation of vector/matrix type");
         }
-        operands[i] = implicit_conversion(operands[i], type, &operands[i]->loc);
-        if (!operands[i])
+
+        if (!(cast = new_cast(operands[i], type, &operands[i]->loc)))
         {
-            ERR("Impossible to convert expression operand %u to %s\n", i + 1, debug_hlsl_type(type));
             d3dcompiler_free(expr);
             return NULL;
         }
+        list_add_after(&operands[i]->entry, &cast->node.entry);
+        operands[i] = &cast->node;
     }
     expr->node.data_type = type;
     expr->op = op;
@@ -1362,8 +1380,8 @@ struct hlsl_ir_deref *new_var_deref(struct hlsl_ir_var *var)
     }
     deref->node.type = HLSL_IR_DEREF;
     deref->node.data_type = var->data_type;
-    deref->type = HLSL_IR_DEREF_VAR;
-    deref->v.var = var;
+    deref->src.type = HLSL_IR_DEREF_VAR;
+    deref->src.v.var = var;
     return deref;
 }
 
@@ -1378,9 +1396,9 @@ struct hlsl_ir_deref *new_record_deref(struct hlsl_ir_node *record, struct hlsl_
     }
     deref->node.type = HLSL_IR_DEREF;
     deref->node.data_type = field->type;
-    deref->type = HLSL_IR_DEREF_RECORD;
-    deref->v.record.record = record;
-    deref->v.record.field = field;
+    deref->src.type = HLSL_IR_DEREF_RECORD;
+    deref->src.v.record.record = record;
+    deref->src.v.record.field = field;
     return deref;
 }
 
@@ -1404,12 +1422,46 @@ static enum hlsl_ir_expr_op op_from_assignment(enum parse_assign_op op)
     return ops[op];
 }
 
-struct hlsl_ir_node *make_assignment(struct hlsl_ir_node *left, enum parse_assign_op assign_op,
-        DWORD writemask, struct hlsl_ir_node *right)
+static unsigned int invert_swizzle(unsigned int *swizzle, unsigned int writemask)
+{
+    unsigned int i, j, bit = 0, inverted = 0, components, new_writemask = 0, new_swizzle = 0;
+
+    /* Apply the writemask to the swizzle to get a new writemask and swizzle. */
+    for (i = 0; i < 4; ++i)
+    {
+        if (writemask & (1 << i))
+        {
+            unsigned int s = (*swizzle >> (i * 2)) & 3;
+            new_swizzle |= s << (bit++ * 2);
+            if (new_writemask & (1 << s))
+                return 0;
+            new_writemask |= 1 << s;
+        }
+    }
+    components = bit;
+
+    /* Invert the swizzle. */
+    bit = 0;
+    for (i = 0; i < 4; ++i)
+    {
+        for (j = 0; j < components; ++j)
+        {
+            unsigned int s = (new_swizzle >> (j * 2)) & 3;
+            if (s == i)
+                inverted |= j << (bit++ * 2);
+        }
+    }
+
+    *swizzle = inverted;
+    return new_writemask;
+}
+
+struct hlsl_ir_node *make_assignment(struct hlsl_ir_node *lhs, enum parse_assign_op assign_op,
+        struct hlsl_ir_node *rhs)
 {
     struct hlsl_ir_assignment *assign = d3dcompiler_alloc(sizeof(*assign));
+    DWORD writemask = (1 << lhs->data_type->dimx) - 1;
     struct hlsl_type *type;
-    struct hlsl_ir_node *lhs, *rhs;
 
     if (!assign)
     {
@@ -1417,100 +1469,110 @@ struct hlsl_ir_node *make_assignment(struct hlsl_ir_node *left, enum parse_assig
         return NULL;
     }
 
+    while (lhs->type != HLSL_IR_DEREF)
+    {
+        struct hlsl_ir_node *lhs_inner;
+
+        if (lhs->type == HLSL_IR_EXPR && expr_from_node(lhs)->op == HLSL_IR_UNOP_CAST)
+        {
+            FIXME("Cast on the lhs.\n");
+            d3dcompiler_free(assign);
+            return NULL;
+        }
+        else if (lhs->type == HLSL_IR_SWIZZLE)
+        {
+            struct hlsl_ir_swizzle *swizzle = swizzle_from_node(lhs);
+
+            if (lhs->data_type->type == HLSL_CLASS_MATRIX)
+                FIXME("Assignments with writemasks and matrices on lhs are not supported yet.\n");
+
+            lhs_inner = swizzle->val;
+            list_remove(&lhs->entry);
+
+            list_add_after(&rhs->entry, &lhs->entry);
+            swizzle->val = rhs;
+            if (!(writemask = invert_swizzle(&swizzle->swizzle, writemask)))
+            {
+                hlsl_report_message(lhs->loc, HLSL_LEVEL_ERROR, "invalid writemask");
+                d3dcompiler_free(assign);
+                return NULL;
+            }
+            rhs = &swizzle->node;
+        }
+        else
+        {
+            hlsl_report_message(lhs->loc, HLSL_LEVEL_ERROR, "invalid lvalue");
+            d3dcompiler_free(assign);
+            return NULL;
+        }
+
+        lhs = lhs_inner;
+    }
+
     TRACE("Creating proper assignment expression.\n");
-    rhs = right;
     if (writemask == BWRITERSP_WRITEMASK_ALL)
-        type = left->data_type;
+        type = lhs->data_type;
     else
     {
         unsigned int dimx = 0;
         DWORD bitmask;
         enum hlsl_type_class type_class;
 
-        if (left->data_type->type > HLSL_CLASS_LAST_NUMERIC)
+        if (lhs->data_type->type > HLSL_CLASS_LAST_NUMERIC)
         {
-            hlsl_report_message(left->loc.file, left->loc.line, left->loc.col, HLSL_LEVEL_ERROR,
+            hlsl_report_message(lhs->loc, HLSL_LEVEL_ERROR,
                     "writemask on a non scalar/vector/matrix type");
             d3dcompiler_free(assign);
             return NULL;
         }
-        bitmask = writemask & ((1 << left->data_type->dimx) - 1);
+        bitmask = writemask & ((1 << lhs->data_type->dimx) - 1);
         while (bitmask)
         {
             if (bitmask & 1)
                 dimx++;
             bitmask >>= 1;
         }
-        if (left->data_type->type == HLSL_CLASS_MATRIX)
+        if (lhs->data_type->type == HLSL_CLASS_MATRIX)
             FIXME("Assignments with writemasks and matrices on lhs are not supported yet.\n");
         if (dimx == 1)
             type_class = HLSL_CLASS_SCALAR;
         else
-            type_class = left->data_type->type;
-        type = new_hlsl_type(NULL, type_class, left->data_type->base_type, dimx, 1);
+            type_class = lhs->data_type->type;
+        type = new_hlsl_type(NULL, type_class, lhs->data_type->base_type, dimx, 1);
     }
     assign->node.type = HLSL_IR_ASSIGNMENT;
-    assign->node.loc = left->loc;
+    assign->node.loc = lhs->loc;
     assign->node.data_type = type;
     assign->writemask = writemask;
-    FIXME("Check for casts in the lhs.\n");
 
-    lhs = left;
-    /* FIXME: check for invalid writemasks on the lhs. */
+    rhs = implicit_conversion(rhs, type, &rhs->loc);
 
-    if (!compare_hlsl_types(type, rhs->data_type))
-    {
-        struct hlsl_ir_node *converted_rhs;
-
-        if (!implicit_compatible_data_types(rhs->data_type, type))
-        {
-            hlsl_report_message(rhs->loc.file, rhs->loc.line, rhs->loc.col, HLSL_LEVEL_ERROR,
-                    "can't implicitly convert %s to %s",
-                    debug_hlsl_type(rhs->data_type), debug_hlsl_type(type));
-            free_instr(lhs);
-            free_instr(rhs);
-            d3dcompiler_free(assign);
-            return NULL;
-        }
-        if (lhs->data_type->dimx * lhs->data_type->dimy < rhs->data_type->dimx * rhs->data_type->dimy)
-            hlsl_report_message(rhs->loc.file, rhs->loc.line, rhs->loc.col, HLSL_LEVEL_WARNING,
-                    "implicit truncation of vector type");
-
-        converted_rhs = implicit_conversion(rhs, type, &rhs->loc);
-        if (!converted_rhs)
-        {
-            ERR("Couldn't implicitly convert expression to %s.\n", debug_hlsl_type(type));
-            free_instr(lhs);
-            free_instr(rhs);
-            d3dcompiler_free(assign);
-            return NULL;
-        }
-        rhs = converted_rhs;
-    }
-
-    assign->lhs = lhs;
+    assign->lhs = deref_from_node(lhs)->src;
     if (assign_op != ASSIGN_OP_ASSIGN)
     {
         enum hlsl_ir_expr_op op = op_from_assignment(assign_op);
         struct hlsl_ir_node *expr;
 
-        if (lhs->type != HLSL_IR_DEREF || deref_from_node(lhs)->type != HLSL_IR_DEREF_VAR)
+        if (assign->lhs.type != HLSL_IR_DEREF_VAR)
         {
             FIXME("LHS expression not supported in compound assignments yet.\n");
             assign->rhs = rhs;
         }
         else
         {
-            struct hlsl_ir_deref *lhs_deref = deref_from_node(lhs), *new_deref;
-
             TRACE("Adding an expression for the compound assignment.\n");
-            new_deref = new_var_deref(lhs_deref->v.var);
-            expr = new_binary_expr(op, &new_deref->node, rhs, left->loc);
+            expr = new_binary_expr(op, lhs, rhs, lhs->loc);
+            list_add_after(&rhs->entry, &expr->entry);
             assign->rhs = expr;
         }
     }
     else
+    {
+        list_remove(&lhs->entry);
+        /* Don't recursively free the deref; we just copied its members. */
+        d3dcompiler_free(lhs);
         assign->rhs = rhs;
+    }
 
     return &assign->node;
 }
@@ -1664,7 +1726,7 @@ void init_functions_tree(struct wine_rb_tree *funcs)
     wine_rb_init(&hlsl_ctx.functions, compare_function_rb);
 }
 
-static const char *debug_base_type(const struct hlsl_type *type)
+const char *debug_base_type(const struct hlsl_type *type)
 {
     const char *name = "(unknown)";
 
@@ -1756,7 +1818,7 @@ const char *debug_modifiers(DWORD modifiers)
     return wine_dbg_sprintf("%s", string[0] ? string + 1 : "");
 }
 
-static const char *debug_node_type(enum hlsl_ir_node_type type)
+const char *debug_node_type(enum hlsl_ir_node_type type)
 {
     static const char * const names[] =
     {
@@ -1766,6 +1828,7 @@ static const char *debug_node_type(enum hlsl_ir_node_type type)
         "HLSL_IR_DEREF",
         "HLSL_IR_EXPR",
         "HLSL_IR_IF",
+        "HLSL_IR_LOOP",
         "HLSL_IR_JUMP",
         "HLSL_IR_SWIZZLE",
     };
@@ -1784,37 +1847,42 @@ static void debug_dump_instr_list(const struct list *list)
     LIST_FOR_EACH_ENTRY(instr, list, struct hlsl_ir_node, entry)
     {
         debug_dump_instr(instr);
-        TRACE("\n");
+        wine_dbg_printf("\n");
     }
 }
 
+static void debug_dump_src(const struct hlsl_ir_node *node)
+{
+    wine_dbg_printf("%p", node);
+}
+
 static void debug_dump_ir_var(const struct hlsl_ir_var *var)
 {
     if (var->modifiers)
-        TRACE("%s ", debug_modifiers(var->modifiers));
-    TRACE("%s %s", debug_hlsl_type(var->data_type), var->name);
+        wine_dbg_printf("%s ", debug_modifiers(var->modifiers));
+    wine_dbg_printf("%s %s", debug_hlsl_type(var->data_type), var->name);
     if (var->semantic)
-        TRACE(" : %s", debugstr_a(var->semantic));
+        wine_dbg_printf(" : %s", debugstr_a(var->semantic));
 }
 
-static void debug_dump_ir_deref(const struct hlsl_ir_deref *deref)
+static void debug_dump_deref(const struct hlsl_deref *deref)
 {
     switch (deref->type)
     {
         case HLSL_IR_DEREF_VAR:
-            TRACE("deref(");
+            wine_dbg_printf("deref(");
             debug_dump_ir_var(deref->v.var);
-            TRACE(")");
+            wine_dbg_printf(")");
             break;
         case HLSL_IR_DEREF_ARRAY:
-            debug_dump_instr(deref->v.array.array);
-            TRACE("[");
-            debug_dump_instr(deref->v.array.index);
-            TRACE("]");
+            debug_dump_src(deref->v.array.array);
+            wine_dbg_printf("[");
+            debug_dump_src(deref->v.array.index);
+            wine_dbg_printf("]");
             break;
         case HLSL_IR_DEREF_RECORD:
-            debug_dump_instr(deref->v.record.record);
-            TRACE(".%s", debugstr_a(deref->v.record.field->name));
+            debug_dump_src(deref->v.record.record);
+            wine_dbg_printf(".%s", debugstr_a(deref->v.record.field->name));
             break;
     }
 }
@@ -1825,39 +1893,39 @@ static void debug_dump_ir_constant(const struct hlsl_ir_constant *constant)
     unsigned int x, y;
 
     if (type->dimy != 1)
-        TRACE("{");
+        wine_dbg_printf("{");
     for (y = 0; y < type->dimy; ++y)
     {
         if (type->dimx != 1)
-            TRACE("{");
+            wine_dbg_printf("{");
         for (x = 0; x < type->dimx; ++x)
         {
             switch (type->base_type)
             {
                 case HLSL_TYPE_FLOAT:
-                    TRACE("%g ", (double)constant->v.value.f[y * type->dimx + x]);
+                    wine_dbg_printf("%g ", (double)constant->v.value.f[y * type->dimx + x]);
                     break;
                 case HLSL_TYPE_DOUBLE:
-                    TRACE("%g ", constant->v.value.d[y * type->dimx + x]);
+                    wine_dbg_printf("%g ", constant->v.value.d[y * type->dimx + x]);
                     break;
                 case HLSL_TYPE_INT:
-                    TRACE("%d ", constant->v.value.i[y * type->dimx + x]);
+                    wine_dbg_printf("%d ", constant->v.value.i[y * type->dimx + x]);
                     break;
                 case HLSL_TYPE_UINT:
-                    TRACE("%u ", constant->v.value.u[y * type->dimx + x]);
+                    wine_dbg_printf("%u ", constant->v.value.u[y * type->dimx + x]);
                     break;
                 case HLSL_TYPE_BOOL:
-                    TRACE("%s ", constant->v.value.b[y * type->dimx + x] == FALSE ? "false" : "true");
+                    wine_dbg_printf("%s ", constant->v.value.b[y * type->dimx + x] == FALSE ? "false" : "true");
                     break;
                 default:
-                    TRACE("Constants of type %s not supported\n", debug_base_type(type));
+                    wine_dbg_printf("Constants of type %s not supported\n", debug_base_type(type));
             }
         }
         if (type->dimx != 1)
-            TRACE("}");
+            wine_dbg_printf("}");
     }
     if (type->dimy != 1)
-        TRACE("}");
+        wine_dbg_printf("}");
 }
 
 static const char *debug_expr_op(const struct hlsl_ir_expr *expr)
@@ -1941,26 +2009,26 @@ static void debug_dump_ir_expr(const struct hlsl_ir_expr *expr)
 {
     unsigned int i;
 
-    TRACE("%s (", debug_expr_op(expr));
+    wine_dbg_printf("%s (", debug_expr_op(expr));
     for (i = 0; i < 3 && expr->operands[i]; ++i)
     {
-        debug_dump_instr(expr->operands[i]);
-        TRACE(" ");
+        debug_dump_src(expr->operands[i]);
+        wine_dbg_printf(" ");
     }
-    TRACE(")");
+    wine_dbg_printf(")");
 }
 
 static void debug_dump_ir_constructor(const struct hlsl_ir_constructor *constructor)
 {
     unsigned int i;
 
-    TRACE("%s (", debug_hlsl_type(constructor->node.data_type));
+    wine_dbg_printf("%s (", debug_hlsl_type(constructor->node.data_type));
     for (i = 0; i < constructor->args_count; ++i)
     {
-        debug_dump_instr(constructor->args[i]);
-        TRACE(" ");
+        debug_dump_src(constructor->args[i]);
+        wine_dbg_printf(" ");
     }
-    TRACE(")");
+    wine_dbg_printf(")");
 }
 
 static const char *debug_writemask(DWORD writemask)
@@ -1984,32 +2052,32 @@ static const char *debug_writemask(DWORD writemask)
 
 static void debug_dump_ir_assignment(const struct hlsl_ir_assignment *assign)
 {
-    TRACE("= (");
-    debug_dump_instr(assign->lhs);
+    wine_dbg_printf("= (");
+    debug_dump_deref(&assign->lhs);
     if (assign->writemask != BWRITERSP_WRITEMASK_ALL)
-        TRACE("%s", debug_writemask(assign->writemask));
-    TRACE(" ");
-    debug_dump_instr(assign->rhs);
-    TRACE(")");
+        wine_dbg_printf("%s", debug_writemask(assign->writemask));
+    wine_dbg_printf(" ");
+    debug_dump_src(assign->rhs);
+    wine_dbg_printf(")");
 }
 
 static void debug_dump_ir_swizzle(const struct hlsl_ir_swizzle *swizzle)
 {
     unsigned int i;
 
-    debug_dump_instr(swizzle->val);
-    TRACE(".");
+    debug_dump_src(swizzle->val);
+    wine_dbg_printf(".");
     if (swizzle->val->data_type->dimy > 1)
     {
         for (i = 0; i < swizzle->node.data_type->dimx; ++i)
-            TRACE("_m%u%u", (swizzle->swizzle >> i * 8) & 0xf, (swizzle->swizzle >> (i * 8 + 4)) & 0xf);
+            wine_dbg_printf("_m%u%u", (swizzle->swizzle >> i * 8) & 0xf, (swizzle->swizzle >> (i * 8 + 4)) & 0xf);
     }
     else
     {
         static const char c[] = {'x', 'y', 'z', 'w'};
 
         for (i = 0; i < swizzle->node.data_type->dimx; ++i)
-            TRACE("%c", c[(swizzle->swizzle >> i * 2) & 0x3]);
+            wine_dbg_printf("%c", c[(swizzle->swizzle >> i * 2) & 0x3]);
     }
 }
 
@@ -2018,47 +2086,55 @@ static void debug_dump_ir_jump(const struct hlsl_ir_jump *jump)
     switch (jump->type)
     {
         case HLSL_IR_JUMP_BREAK:
-            TRACE("break");
+            wine_dbg_printf("break");
             break;
         case HLSL_IR_JUMP_CONTINUE:
-            TRACE("continue");
+            wine_dbg_printf("continue");
             break;
         case HLSL_IR_JUMP_DISCARD:
-            TRACE("discard");
+            wine_dbg_printf("discard");
             break;
         case HLSL_IR_JUMP_RETURN:
-            TRACE("return ");
+            wine_dbg_printf("return ");
             if (jump->return_value)
-                debug_dump_instr(jump->return_value);
-            TRACE(";");
+                debug_dump_src(jump->return_value);
+            wine_dbg_printf(";");
             break;
     }
 }
 
 static void debug_dump_ir_if(const struct hlsl_ir_if *if_node)
 {
-    TRACE("if (");
-    debug_dump_instr(if_node->condition);
-    TRACE(")\n{\n");
+    wine_dbg_printf("if (");
+    debug_dump_src(if_node->condition);
+    wine_dbg_printf(")\n{\n");
     debug_dump_instr_list(if_node->then_instrs);
-    TRACE("}\n");
+    wine_dbg_printf("}\n");
     if (if_node->else_instrs)
     {
-        TRACE("else\n{\n");
+        wine_dbg_printf("else\n{\n");
         debug_dump_instr_list(if_node->else_instrs);
-        TRACE("}\n");
+        wine_dbg_printf("}\n");
     }
 }
 
+static void debug_dump_ir_loop(const struct hlsl_ir_loop *loop)
+{
+    wine_dbg_printf("for (;;)\n{\n");
+    debug_dump_instr_list(loop->body);
+    wine_dbg_printf("}\n");
+}
+
 static void debug_dump_instr(const struct hlsl_ir_node *instr)
 {
+    wine_dbg_printf("%p: ", instr);
     switch (instr->type)
     {
         case HLSL_IR_EXPR:
             debug_dump_ir_expr(expr_from_node(instr));
             break;
         case HLSL_IR_DEREF:
-            debug_dump_ir_deref(deref_from_node(instr));
+            debug_dump_deref(&deref_from_node(instr)->src);
             break;
         case HLSL_IR_CONSTANT:
             debug_dump_ir_constant(constant_from_node(instr));
@@ -2078,8 +2154,11 @@ static void debug_dump_instr(const struct hlsl_ir_node *instr)
         case HLSL_IR_IF:
             debug_dump_ir_if(if_from_node(instr));
             break;
+        case HLSL_IR_LOOP:
+            debug_dump_ir_loop(loop_from_node(instr));
+            break;
         default:
-            TRACE("<No dump function for %s>", debug_node_type(instr->type));
+            wine_dbg_printf("<No dump function for %s>", debug_node_type(instr->type));
     }
 }
 
@@ -2092,7 +2171,7 @@ void debug_dump_ir_function_decl(const struct hlsl_ir_function_decl *func)
     LIST_FOR_EACH_ENTRY(param, func->parameters, struct hlsl_ir_var, param_entry)
     {
         debug_dump_ir_var(param);
-        TRACE("\n");
+        wine_dbg_printf("\n");
     }
     if (func->semantic)
         TRACE("Function semantic: %s\n", debugstr_a(func->semantic));
@@ -2155,69 +2234,44 @@ static void free_ir_constant(struct hlsl_ir_constant *constant)
 
 static void free_ir_deref(struct hlsl_ir_deref *deref)
 {
-    switch (deref->type)
-    {
-        case HLSL_IR_DEREF_VAR:
-            /* Variables are shared among nodes in the tree. */
-            break;
-        case HLSL_IR_DEREF_ARRAY:
-            free_instr(deref->v.array.array);
-            free_instr(deref->v.array.index);
-            break;
-        case HLSL_IR_DEREF_RECORD:
-            free_instr(deref->v.record.record);
-            break;
-    }
     d3dcompiler_free(deref);
 }
 
 static void free_ir_swizzle(struct hlsl_ir_swizzle *swizzle)
 {
-    free_instr(swizzle->val);
     d3dcompiler_free(swizzle);
 }
 
 static void free_ir_constructor(struct hlsl_ir_constructor *constructor)
 {
-    unsigned int i;
-    for (i = 0; i < constructor->args_count; ++i)
-        free_instr(constructor->args[i]);
     d3dcompiler_free(constructor);
 }
 
 static void free_ir_expr(struct hlsl_ir_expr *expr)
 {
-    unsigned int i;
-
-    for (i = 0; i < 3; ++i)
-    {
-        if (!expr->operands[i])
-            break;
-        free_instr(expr->operands[i]);
-    }
-    free_instr_list(expr->subexpressions);
     d3dcompiler_free(expr);
 }
 
 static void free_ir_assignment(struct hlsl_ir_assignment *assignment)
 {
-    free_instr(assignment->lhs);
-    free_instr(assignment->rhs);
     d3dcompiler_free(assignment);
 }
 
 static void free_ir_if(struct hlsl_ir_if *if_node)
 {
-    free_instr(if_node->condition);
     free_instr_list(if_node->then_instrs);
     free_instr_list(if_node->else_instrs);
     d3dcompiler_free(if_node);
 }
 
+static void free_ir_loop(struct hlsl_ir_loop *loop)
+{
+    free_instr_list(loop->body);
+    d3dcompiler_free(loop);
+}
+
 static void free_ir_jump(struct hlsl_ir_jump *jump)
 {
-    if (jump->type == HLSL_IR_JUMP_RETURN)
-        free_instr(jump->return_value);
     d3dcompiler_free(jump);
 }
 
@@ -2246,6 +2300,9 @@ void free_instr(struct hlsl_ir_node *node)
         case HLSL_IR_IF:
             free_ir_if(if_from_node(node));
             break;
+        case HLSL_IR_LOOP:
+            free_ir_loop(loop_from_node(node));
+            break;
         case HLSL_IR_JUMP:
             free_ir_jump(jump_from_node(node));
             break;
@@ -2327,3 +2384,4 @@ void add_function_decl(struct wine_rb_tree *funcs, char *name, struct hlsl_ir_fu
     func->intrinsic = intrinsic;
     wine_rb_put(funcs, func->name, &func->entry);
 }
+#endif
