@@ -358,6 +358,7 @@ struct module
     enum module_type		type : 16;
     unsigned short              is_virtual : 1;
     DWORD64                     reloc_delta;
+    WCHAR*                      real_path;
 
     /* specific information for debug types */
     struct module_format*       format_info[DFI_LAST];
@@ -587,8 +588,6 @@ extern BOOL         pcs_callback(const struct process* pcs, ULONG action, void*
 extern void*        fetch_buffer(struct process* pcs, unsigned size) DECLSPEC_HIDDEN;
 extern const char*  wine_dbgstr_addr(const ADDRESS64* addr) DECLSPEC_HIDDEN;
 extern struct cpu*  cpu_find(DWORD) DECLSPEC_HIDDEN;
-
-/* crc32.c */
 extern DWORD calc_crc32(int fd) DECLSPEC_HIDDEN;
 
 typedef BOOL (*enum_modules_cb)(const WCHAR*, unsigned long addr, void* user);
