@@ -134,9 +134,8 @@ static void module_fill_module(const WCHAR* in, WCHAR* out, size_t size)
 
 void module_set_module(struct module* module, const WCHAR* name)
 {
-    module_fill_module(name, module->module.ModuleName,
-            sizeof(module->module.ModuleName) / sizeof(module->module.ModuleName[0]));
-    module_fill_module(name, module->modulename, sizeof(module->modulename) / sizeof(module->modulename[0]));
+    module_fill_module(name, module->module.ModuleName, ARRAY_SIZE(module->module.ModuleName));
+    module_fill_module(name, module->modulename, ARRAY_SIZE(module->modulename));
 }
 
 /* Returned string must be freed by caller */
@@ -215,7 +214,7 @@ struct module* module_new(struct process* pcs, const WCHAR* name,
     module->module.ImageSize = size;
     module_set_module(module, name);
     module->module.ImageName[0] = '\0';
-    lstrcpynW(module->module.LoadedImageName, name, sizeof(module->module.LoadedImageName) / sizeof(WCHAR));
+    lstrcpynW(module->module.LoadedImageName, name, ARRAY_SIZE(module->module.LoadedImageName));
     module->module.SymType = SymNone;
     module->module.NumSyms = 0;
     module->module.TimeDateStamp = stamp;
@@ -281,7 +280,7 @@ struct module* module_find_by_nameA(const struct process* pcs, const char* name)
 {
     WCHAR wname[MAX_PATH];
 
-    MultiByteToWideChar(CP_ACP, 0, name, -1, wname, sizeof(wname) / sizeof(WCHAR));
+    MultiByteToWideChar(CP_ACP, 0, name, -1, wname, ARRAY_SIZE(wname));
     return module_find_by_nameW(pcs, wname);
 }
 
@@ -335,16 +334,15 @@ static struct module* module_get_container(const struct process* pcs,
  *           module_get_containee
  *
  */
-struct module* module_get_containee(const struct process* pcs, 
-                                    const struct module* outter)
+struct module* module_get_containee(const struct process* pcs, const struct module* outer)
 {
     struct module*      module;
-     
+
     for (module = pcs->lmodules; module; module = module->next)
     {
-        if (module != outter &&
-            outter->module.BaseOfImage <= module->module.BaseOfImage &&
-            outter->module.BaseOfImage + outter->module.ImageSize >=
+        if (module != outer &&
+            outer->module.BaseOfImage <= module->module.BaseOfImage &&
+            outer->module.BaseOfImage + outer->module.ImageSize >=
             module->module.BaseOfImage + module->module.ImageSize)
             return module;
     }
@@ -397,7 +395,7 @@ BOOL module_get_debug(struct module_pair* pair)
                          &idslW64);
             break;
         case DMT_MACHO:
-            ret = macho_load_debug_info(pair->effective);
+            ret = macho_load_debug_info(pair->pcs, pair->effective);
             break;
         default:
             ret = FALSE;
@@ -437,7 +435,7 @@ struct module* module_find_by_addr(const struct process* pcs, DWORD64 addr,
                 return module;
         }
     }
-    SetLastError(ERROR_INVALID_ADDRESS);
+    SetLastError(ERROR_MOD_NOT_FOUND);
     return module;
 }
 
@@ -671,8 +669,7 @@ DWORD64 WINAPI  SymLoadModuleExW(HANDLE hProcess, HANDLE hFile, PCWSTR wImageNam
     if (wModuleName)
         module_set_module(module, wModuleName);
     if (wImageName)
-        lstrcpynW(module->module.ImageName, wImageName,
-              sizeof(module->module.ImageName) / sizeof(WCHAR));
+        lstrcpynW(module->module.ImageName, wImageName, ARRAY_SIZE(module->module.ImageName));
 
     return module->module.BaseOfImage;
 }
@@ -708,6 +705,7 @@ BOOL module_remove(struct process* pcs, struct module* module)
     hash_table_destroy(&module->ht_types);
     HeapFree(GetProcessHeap(), 0, module->sources);
     HeapFree(GetProcessHeap(), 0, module->addr_sorttab);
+    HeapFree(GetProcessHeap(), 0, module->real_path);
     pool_destroy(&module->pool);
     /* native dbghelp doesn't invoke registered callback(,CBA_SYMBOLS_UNLOADED,) here
      * so do we
@@ -835,7 +833,7 @@ BOOL  WINAPI SymEnumerateModulesW64(HANDLE hProcess,
     
     for (module = pcs->lmodules; module; module = module->next)
     {
-        if (!(dbghelp_options & SYMOPT_WINE_WITH_NATIVE_MODULES) &&
+        if (!dbghelp_opt_native &&
             (module->type == DMT_ELF || module->type == DMT_MACHO))
             continue;
         if (!EnumModulesCallback(module->modulename,
@@ -935,9 +933,9 @@ BOOL  WINAPI EnumerateLoadedModulesW64(HANDLE hProcess,
     for (i = 0; i < sz; i++)
     {
         if (!GetModuleInformation(hProcess, hMods[i], &mi, sizeof(mi)) ||
-            !GetModuleBaseNameW(hProcess, hMods[i], baseW, sizeof(baseW) / sizeof(WCHAR)))
+            !GetModuleBaseNameW(hProcess, hMods[i], baseW, ARRAY_SIZE(baseW)))
             continue;
-        module_fill_module(baseW, modW, sizeof(modW) / sizeof(modW[0]));
+        module_fill_module(baseW, modW, ARRAY_SIZE(modW));
         EnumLoadedModulesCallback(modW, (DWORD_PTR)mi.lpBaseOfDll, mi.SizeOfImage,
                                   UserContext);
     }
