@@ -229,6 +229,7 @@ static BOOL do_searchW(PCWSTR file, PWSTR buffer, BOOL recurse,
     static const WCHAR  S_DotDotW[] = {'.','.','\0'};
 
     pos = strlenW(buffer);
+    if (pos == 0) return FALSE;
     if (buffer[pos - 1] != '\\') buffer[pos++] = '\\';
     strcpyW(buffer + pos, S_AllW);
     if ((h = FindFirstFileW(buffer, &fd)) == INVALID_HANDLE_VALUE)
@@ -663,6 +664,11 @@ BOOL path_find_symbol_file(const struct process* pcs, const struct module* modul
     /* 2. check module-path */
     file_pathW(module->module.LoadedImageName, buffer);
     if (do_searchW(filename, buffer, FALSE, module_find_cb, &mf)) return TRUE;
+    if (module->real_path)
+    {
+        file_pathW(module->real_path, buffer);
+        if (do_searchW(filename, buffer, FALSE, module_find_cb, &mf)) return TRUE;
+    }
 
     while (searchPath)
     {
