@@ -49,8 +49,6 @@ static void dxgi_mode1_from_wined3d(DXGI_MODE_DESC1 *mode, const struct wined3d_
 static HRESULT dxgi_output_find_closest_matching_mode(struct dxgi_output *output,
         struct wined3d_display_mode *mode, IUnknown *device)
 {
-    struct dxgi_adapter *adapter;
-    struct wined3d *wined3d;
     HRESULT hr;
 
     if (!mode->width != !mode->height)
@@ -66,15 +64,31 @@ static HRESULT dxgi_output_find_closest_matching_mode(struct dxgi_output *output
     }
 
     wined3d_mutex_lock();
-    adapter = output->adapter;
-    wined3d = adapter->factory->wined3d;
-
-    hr = wined3d_find_closest_matching_adapter_mode(wined3d, adapter->ordinal, mode);
+    hr = wined3d_output_find_closest_matching_mode(output->wined3d_output, mode);
     wined3d_mutex_unlock();
 
     return hr;
 }
 
+static int dxgi_mode_desc_compare(const void *l, const void *r)
+{
+    const DXGI_MODE_DESC *left = l, *right = r;
+    int a, b;
+
+    if (left->Width != right->Width)
+        return left->Width - right->Width;
+
+    if (left->Height != right->Height)
+        return left->Height - right->Height;
+
+    a = left->RefreshRate.Numerator * right->RefreshRate.Denominator;
+    b = right->RefreshRate.Numerator * left->RefreshRate.Denominator;
+    if (a != b)
+        return a - b;
+
+    return 0;
+}
+
 enum dxgi_mode_struct_version
 {
     DXGI_MODE_STRUCT_VERSION_0,
@@ -88,7 +102,6 @@ static HRESULT dxgi_output_get_display_mode_list(struct dxgi_output *output,
     enum wined3d_format_id wined3d_format;
     struct wined3d_display_mode mode;
     unsigned int i, max_count;
-    struct wined3d *wined3d;
     HRESULT hr;
 
     if (!mode_count)
@@ -103,8 +116,7 @@ static HRESULT dxgi_output_get_display_mode_list(struct dxgi_output *output,
     wined3d_format = wined3dformat_from_dxgi_format(format);
 
     wined3d_mutex_lock();
-    wined3d = output->adapter->factory->wined3d;
-    max_count = wined3d_get_adapter_mode_count(wined3d, output->adapter->ordinal,
+    max_count = wined3d_output_get_mode_count(output->wined3d_output,
             wined3d_format, WINED3D_SCANLINE_ORDERING_UNKNOWN);
 
     if (!modes)
@@ -124,10 +136,10 @@ static HRESULT dxgi_output_get_display_mode_list(struct dxgi_output *output,
 
     for (i = 0; i < *mode_count; ++i)
     {
-        if (FAILED(hr = wined3d_enum_adapter_modes(wined3d, output->adapter->ordinal,
-                wined3d_format, WINED3D_SCANLINE_ORDERING_UNKNOWN, i, &mode)))
+        if (FAILED(hr = wined3d_output_get_mode(output->wined3d_output, wined3d_format,
+                WINED3D_SCANLINE_ORDERING_UNKNOWN, i, &mode)))
         {
-            WARN("Failed to enum adapter mode %u, hr %#x.\n", i, hr);
+            WARN("Failed to get output mode %u, hr %#x.\n", i, hr);
             wined3d_mutex_unlock();
             return hr;
         }
@@ -151,6 +163,16 @@ static HRESULT dxgi_output_get_display_mode_list(struct dxgi_output *output,
     }
     wined3d_mutex_unlock();
 
+    switch (struct_version)
+    {
+        case DXGI_MODE_STRUCT_VERSION_0:
+            qsort(modes, *mode_count, sizeof(DXGI_MODE_DESC), dxgi_mode_desc_compare);
+            break;
+        case DXGI_MODE_STRUCT_VERSION_1:
+            qsort(modes, *mode_count, sizeof(DXGI_MODE_DESC1), dxgi_mode_desc_compare);
+            break;
+    }
+
     return S_OK;
 }
 
@@ -335,14 +357,30 @@ static HRESULT STDMETHODCALLTYPE dxgi_output_WaitForVBlank(IDXGIOutput4 *iface)
 
 static HRESULT STDMETHODCALLTYPE dxgi_output_TakeOwnership(IDXGIOutput4 *iface, IUnknown *device, BOOL exclusive)
 {
-    FIXME("iface %p, device %p, exclusive %d stub!\n", iface, device, exclusive);
+    struct dxgi_output *output = impl_from_IDXGIOutput4(iface);
+    HRESULT hr;
 
-    return E_NOTIMPL;
+    TRACE("iface %p, device %p, exclusive %d.\n", iface, device, exclusive);
+
+    if (!device)
+        return DXGI_ERROR_INVALID_CALL;
+
+    wined3d_mutex_lock();
+    hr = wined3d_output_take_ownership(output->wined3d_output, exclusive);
+    wined3d_mutex_unlock();
+
+    return hr;
 }
 
 static void STDMETHODCALLTYPE dxgi_output_ReleaseOwnership(IDXGIOutput4 *iface)
 {
-    FIXME("iface %p stub!\n", iface);
+    struct dxgi_output *output = impl_from_IDXGIOutput4(iface);
+
+    TRACE("iface %p.\n", iface);
+
+    wined3d_mutex_lock();
+    wined3d_output_release_ownership(output->wined3d_output);
+    wined3d_mutex_unlock();
 }
 
 static HRESULT STDMETHODCALLTYPE dxgi_output_GetGammaControlCapabilities(IDXGIOutput4 *iface,
@@ -530,20 +568,23 @@ struct dxgi_output *unsafe_impl_from_IDXGIOutput(IDXGIOutput *iface)
     return CONTAINING_RECORD(iface, struct dxgi_output, IDXGIOutput4_iface);
 }
 
-static void dxgi_output_init(struct dxgi_output *output, struct dxgi_adapter *adapter)
+static void dxgi_output_init(struct dxgi_output *output, unsigned int output_idx,
+        struct dxgi_adapter *adapter)
 {
     output->IDXGIOutput4_iface.lpVtbl = &dxgi_output_vtbl;
     output->refcount = 1;
+    output->wined3d_output = wined3d_adapter_get_output(adapter->wined3d_adapter, output_idx);
     wined3d_private_store_init(&output->private_store);
     output->adapter = adapter;
     IWineDXGIAdapter_AddRef(&output->adapter->IWineDXGIAdapter_iface);
 }
 
-HRESULT dxgi_output_create(struct dxgi_adapter *adapter, struct dxgi_output **output)
+HRESULT dxgi_output_create(struct dxgi_adapter *adapter, unsigned int output_idx,
+        struct dxgi_output **output)
 {
     if (!(*output = heap_alloc_zero(sizeof(**output))))
         return E_OUTOFMEMORY;
 
-    dxgi_output_init(*output, adapter);
+    dxgi_output_init(*output, output_idx, adapter);
     return S_OK;
 }
