@@ -37,6 +37,7 @@ typedef struct {
 struct _function_vtbl_t {
     HRESULT (*call)(script_ctx_t*,FunctionInstance*,IDispatch*,unsigned,unsigned,jsval_t*,jsval_t*);
     HRESULT (*toString)(FunctionInstance*,jsstr_t**);
+    function_code_t* (*get_code)(FunctionInstance*);
     void (*destructor)(FunctionInstance*);
 };
 
@@ -291,7 +292,7 @@ static HRESULT Function_toString(script_ctx_t *ctx, vdisp_t *jsthis, WORD flags,
     TRACE("\n");
 
     if(!(function = function_this(jsthis)))
-        return throw_type_error(ctx, JS_E_FUNCTION_EXPECTED, NULL);
+        return JS_E_FUNCTION_EXPECTED;
 
     hres = function->vtbl->toString(function, &str);
     if(FAILED(hres))
@@ -351,7 +352,7 @@ static HRESULT Function_apply(script_ctx_t *ctx, vdisp_t *jsthis, WORD flags, un
     TRACE("\n");
 
     if(!(function = function_this(jsthis)) && (jsthis->flags & VDISP_JSDISP))
-        return throw_type_error(ctx, JS_E_FUNCTION_EXPECTED, NULL);
+        return JS_E_FUNCTION_EXPECTED;
 
     if(argc) {
         if(!is_undefined(argv[0]) && !is_null(argv[0])) {
@@ -416,7 +417,7 @@ static HRESULT Function_call(script_ctx_t *ctx, vdisp_t *jsthis, WORD flags, uns
     TRACE("\n");
 
     if(!(function = function_this(jsthis)))
-        return throw_type_error(ctx, JS_E_FUNCTION_EXPECTED, NULL);
+        return JS_E_FUNCTION_EXPECTED;
 
     if(argc) {
         if(!is_undefined(argv[0]) && !is_null(argv[0])) {
@@ -445,7 +446,7 @@ static HRESULT Function_bind(script_ctx_t *ctx, vdisp_t *jsthis, WORD flags, uns
     TRACE("\n");
 
     if(!(function = function_this(jsthis)))
-        return throw_type_error(ctx, JS_E_FUNCTION_EXPECTED, NULL);
+        return JS_E_FUNCTION_EXPECTED;
 
     if(argc < 1) {
         FIXME("no this argument\n");
@@ -524,6 +525,16 @@ static HRESULT Function_get_arguments(script_ctx_t *ctx, jsdisp_t *jsthis, jsval
     return S_OK;
 }
 
+function_code_t *Function_get_code(jsdisp_t *jsthis)
+{
+    FunctionInstance *function;
+
+    assert(is_class(jsthis, JSCLASS_FUNCTION));
+    function = function_from_jsdisp(jsthis);
+
+    return function->vtbl->get_code(function);
+}
+
 static void Function_destructor(jsdisp_t *dispex)
 {
     FunctionInstance *function = function_from_jsdisp(dispex);
@@ -601,10 +612,8 @@ static HRESULT NativeFunction_call(script_ctx_t *ctx, FunctionInstance *func, ID
 
     if(this_disp)
         set_disp(&vthis, this_disp);
-    else if(ctx->host_global)
-        set_disp(&vthis, ctx->host_global);
     else
-        set_jsdisp(&vthis, ctx->global);
+        set_disp(&vthis, lookup_global_host(ctx));
 
     hres = function->proc(ctx, &vthis, flags & ~DISPATCH_JSCRIPT_INTERNAL_MASK, argc, argv, r);
 
@@ -638,6 +647,11 @@ static HRESULT NativeFunction_toString(FunctionInstance *func, jsstr_t **ret)
     return S_OK;
 }
 
+static function_code_t *NativeFunction_get_code(FunctionInstance *function)
+{
+    return NULL;
+}
+
 static void NativeFunction_destructor(FunctionInstance *function)
 {
 }
@@ -645,6 +659,7 @@ static void NativeFunction_destructor(FunctionInstance *function)
 static const function_vtbl_t NativeFunctionVtbl = {
     NativeFunction_call,
     NativeFunction_toString,
+    NativeFunction_get_code,
     NativeFunction_destructor
 };
 
@@ -749,6 +764,13 @@ static HRESULT InterpretedFunction_toString(FunctionInstance *func, jsstr_t **re
     return *ret ? S_OK : E_OUTOFMEMORY;
 }
 
+static function_code_t *InterpretedFunction_get_code(FunctionInstance *func)
+{
+    InterpretedFunction *function = (InterpretedFunction*)func;
+
+    return function->func_code;
+}
+
 static void InterpretedFunction_destructor(FunctionInstance *func)
 {
     InterpretedFunction *function = (InterpretedFunction*)func;
@@ -761,6 +783,7 @@ static void InterpretedFunction_destructor(FunctionInstance *func)
 static const function_vtbl_t InterpretedFunctionVtbl = {
     InterpretedFunction_call,
     InterpretedFunction_toString,
+    InterpretedFunction_get_code,
     InterpretedFunction_destructor
 };
 
@@ -842,6 +865,11 @@ static HRESULT BindFunction_toString(FunctionInstance *function, jsstr_t **ret)
     return *ret ? S_OK : E_OUTOFMEMORY;
 }
 
+static function_code_t *BindFunction_get_code(FunctionInstance *function)
+{
+    return NULL;
+}
+
 static void BindFunction_destructor(FunctionInstance *func)
 {
     BindFunction *function = (BindFunction*)func;
@@ -858,6 +886,7 @@ static void BindFunction_destructor(FunctionInstance *func)
 static const function_vtbl_t BindFunctionVtbl = {
     BindFunction_call,
     BindFunction_toString,
+    BindFunction_get_code,
     BindFunction_destructor
 };
 
@@ -953,7 +982,8 @@ static HRESULT construct_function(script_ctx_t *ctx, unsigned argc, jsval_t *arg
     if(FAILED(hres))
         return hres;
 
-    hres = compile_script(ctx, str, NULL, NULL, FALSE, FALSE, &code);
+    hres = compile_script(ctx, str, 0, 0, NULL, NULL, FALSE, FALSE,
+                          ctx->call_ctx ? ctx->call_ctx->bytecode->named_item : NULL, &code);
     heap_free(str);
     if(FAILED(hres))
         return hres;
