@@ -90,11 +90,6 @@ static BOOL (WINAPI *pIsWow64Process)(HANDLE,PBOOL);
 static BOOL (WINAPI *pSetThreadErrorMode)(DWORD,PDWORD);
 static DWORD (WINAPI *pGetThreadErrorMode)(void);
 static DWORD (WINAPI *pRtlGetThreadErrorMode)(void);
-static BOOL   (WINAPI *pActivateActCtx)(HANDLE,ULONG_PTR*);
-static HANDLE (WINAPI *pCreateActCtxW)(PCACTCTXW);
-static BOOL   (WINAPI *pDeactivateActCtx)(DWORD,ULONG_PTR);
-static BOOL   (WINAPI *pGetCurrentActCtx)(HANDLE *);
-static void   (WINAPI *pReleaseActCtx)(HANDLE);
 static PTP_POOL (WINAPI *pCreateThreadpool)(PVOID);
 static void (WINAPI *pCloseThreadpool)(PTP_POOL);
 static PTP_WORK (WINAPI *pCreateThreadpoolWork)(PTP_WORK_CALLBACK,PVOID,PTP_CALLBACK_ENVIRON);
@@ -312,7 +307,7 @@ static DWORD WINAPI thread_actctx_func(void *p)
     BOOL ret;
 
     cur = (void*)0xdeadbeef;
-    ret = pGetCurrentActCtx(&cur);
+    ret = GetCurrentActCtx(&cur);
     ok(ret, "thread GetCurrentActCtx failed, %u\n", GetLastError());
     ok(cur == param->handle, "got %p, expected %p\n", cur, param->handle);
     param->thread_context = cur;
@@ -1745,14 +1740,16 @@ static unsigned int get_thread_fpu_cw( unsigned long *fpu_cw )
 
 static void test_thread_fpu_cw(void)
 {
-    static const struct { unsigned int cw; unsigned long fpu_cw; } expected_cw[6] =
+    static const struct {
+        unsigned int cw; unsigned long fpu_cw; unsigned long fpu_cw_broken;
+    } expected_cw[6] =
     {
 #ifdef __i386__
         { _MCW_EM | _PC_53, MAKELONG( 0x27f, 0x1f80 ) },
         { _MCW_EM | _PC_53, MAKELONG( 0x27f, 0x1f80 ) },
-        { _EM_INEXACT | _RC_CHOP | _PC_24, MAKELONG( 0xc60, 0x7000 ) },
+        { _EM_INEXACT | _RC_CHOP | _PC_24, MAKELONG( 0xc60, 0x7000 ), MAKELONG( 0xc60, 0x1f80 ) },
         { _MCW_EM | _PC_53, MAKELONG( 0x27f, 0x1f80 ) },
-        { _EM_INEXACT | _RC_CHOP | _PC_24, MAKELONG( 0xc60, 0x7000 ) },
+        { _EM_INEXACT | _RC_CHOP | _PC_24, MAKELONG( 0xc60, 0x7000 ), MAKELONG( 0xc60, 0x1f80 ) },
         { _MCW_EM | _PC_53, MAKELONG( 0x27f, 0x1f80 ) }
 #elif defined(__x86_64__)
         { _MCW_EM | _PC_64, MAKELONG( 0x27f, 0x1f80 ) },
@@ -1788,7 +1785,9 @@ static void test_thread_fpu_cw(void)
     cw = _control87( 0, 0 );
     fpu_cw = get_fpu_cw();
     ok(cw == expected_cw[2].cw, "expected %#x got %#x\n", expected_cw[2].cw, cw);
-    ok(fpu_cw == expected_cw[2].fpu_cw, "expected %#lx got %#lx\n", expected_cw[2].fpu_cw, fpu_cw);
+    ok(fpu_cw == expected_cw[2].fpu_cw ||
+            broken(expected_cw[2].fpu_cw_broken && fpu_cw == expected_cw[2].fpu_cw_broken),
+        "expected %#lx got %#lx\n", expected_cw[2].fpu_cw, fpu_cw);
 
     cw = get_thread_fpu_cw( &fpu_cw );
     ok(cw == expected_cw[3].cw, "expected %#x got %#x\n", expected_cw[3].cw, cw);
@@ -1797,7 +1796,9 @@ static void test_thread_fpu_cw(void)
     cw = _control87( 0, 0 );
     fpu_cw = get_fpu_cw();
     ok(cw == expected_cw[4].cw, "expected %#x got %#x\n", expected_cw[4].cw, cw);
-    ok(fpu_cw == expected_cw[4].fpu_cw, "expected %#lx got %#lx\n", expected_cw[4].fpu_cw, fpu_cw);
+    ok(fpu_cw == expected_cw[4].fpu_cw ||
+            broken(expected_cw[4].fpu_cw_broken && fpu_cw == expected_cw[4].fpu_cw_broken),
+        "expected %#lx got %#lx\n", expected_cw[4].fpu_cw, fpu_cw);
 
     _control87( initial_cw, _MCW_EM | _MCW_RC | _MCW_PC );
     cw = _control87( 0, 0 );
@@ -1846,7 +1847,7 @@ static HANDLE test_create(const char *file)
     actctx.cbSize = sizeof(ACTCTXW);
     actctx.lpSource = path;
 
-    handle = pCreateActCtxW(&actctx);
+    handle = CreateActCtxW(&actctx);
     ok(handle != INVALID_HANDLE_VALUE, "failed to create context, error %u\n", GetLastError());
 
     ok(actctx.cbSize == sizeof(actctx), "cbSize=%d\n", actctx.cbSize);
@@ -1870,12 +1871,6 @@ static void test_thread_actctx(void)
     DWORD tid, ret;
     BOOL b;
 
-    if (!pActivateActCtx)
-    {
-        win_skip("skipping activation context tests\n");
-        return;
-    }
-
     create_manifest_file("testdep1.manifest", manifest_dep);
     create_manifest_file("main.manifest", manifest_main);
 
@@ -1884,7 +1879,7 @@ static void test_thread_actctx(void)
     DeleteFileA("main.manifest");
 
     handle = (void*)0xdeadbeef;
-    b = pGetCurrentActCtx(&handle);
+    b = GetCurrentActCtx(&handle);
     ok(b, "GetCurrentActCtx failed: %u\n", GetLastError());
     ok(handle == 0, "active context %p\n", handle);
 
@@ -1899,17 +1894,17 @@ static void test_thread_actctx(void)
     ok(param.thread_context == NULL, "got wrong thread context %p\n", param.thread_context);
     CloseHandle(thread);
 
-    b = pActivateActCtx(context, &cookie);
+    b = ActivateActCtx(context, &cookie);
     ok(b, "activation failed: %u\n", GetLastError());
 
     handle = 0;
-    b = pGetCurrentActCtx(&handle);
+    b = GetCurrentActCtx(&handle);
     ok(b, "GetCurrentActCtx failed: %u\n", GetLastError());
     ok(handle != 0, "no active context\n");
-    pReleaseActCtx(handle);
+    ReleaseActCtx(handle);
 
     param.handle = NULL;
-    b = pGetCurrentActCtx(&param.handle);
+    b = GetCurrentActCtx(&param.handle);
     ok(b && param.handle != NULL, "failed to get context, %u\n", GetLastError());
 
     param.thread_context = (void*)0xdeadbeef;
@@ -1919,7 +1914,7 @@ static void test_thread_actctx(void)
     ret = WaitForSingleObject(thread, 1000);
     ok(ret == WAIT_OBJECT_0, "wait timeout\n");
     ok(param.thread_context == context, "got wrong thread context %p, %p\n", param.thread_context, context);
-    pReleaseActCtx(param.thread_context);
+    ReleaseActCtx(param.thread_context);
     CloseHandle(thread);
 
     /* similar test for CreateRemoteThread() */
@@ -1930,17 +1925,16 @@ static void test_thread_actctx(void)
     ret = WaitForSingleObject(thread, 1000);
     ok(ret == WAIT_OBJECT_0, "wait timeout\n");
     ok(param.thread_context == context, "got wrong thread context %p, %p\n", param.thread_context, context);
-    pReleaseActCtx(param.thread_context);
+    ReleaseActCtx(param.thread_context);
     CloseHandle(thread);
 
-    pReleaseActCtx(param.handle);
+    ReleaseActCtx(param.handle);
 
-    b = pDeactivateActCtx(0, cookie);
+    b = DeactivateActCtx(0, cookie);
     ok(b, "DeactivateActCtx failed: %u\n", GetLastError());
-    pReleaseActCtx(context);
+    ReleaseActCtx(context);
 }
 
-
 static void WINAPI threadpool_workcallback(PTP_CALLBACK_INSTANCE instance, void *context, PTP_WORK work) {
     int *foo = (int*)context;
 
@@ -2145,14 +2139,15 @@ static void test_thread_description(void)
     ok(len == sizeof(*thread_desc), "Unexpected structure length %u.\n", len);
 
     len2 = 0;
-    thread_desc->Length = 1;
-    thread_desc->Description = (WCHAR *)thread_desc;
+    thread_desc->Description.Length = 1;
+    thread_desc->Description.MaximumLength = 0;
+    thread_desc->Description.Buffer = (WCHAR *)thread_desc;
     status = pNtQueryInformationThread(GetCurrentThread(), ThreadDescription, thread_desc, len, &len2);
     ok(!status, "Failed to get thread info, status %#x.\n", status);
     ok(len2 == sizeof(*thread_desc), "Unexpected structure length %u.\n", len);
-    ok(!thread_desc->Length, "Unexpected description length %#x.\n", thread_desc->Length);
-    ok(thread_desc->Description == (WCHAR *)(thread_desc + 1), "Unexpected description string pointer %p, %p.\n",
-            thread_desc->Description, thread_desc);
+    ok(!thread_desc->Description.Length, "Unexpected description length %#x.\n", thread_desc->Description.Length);
+    ok(thread_desc->Description.Buffer == (WCHAR *)(thread_desc + 1),
+            "Unexpected description string pointer %p, %p.\n", thread_desc->Description.Buffer, thread_desc);
 
     hr = pSetThreadDescription(GetCurrentThread(), NULL);
     ok(hr == HRESULT_FROM_NT(STATUS_SUCCESS), "Failed to set thread description, hr %#x.\n", hr);
@@ -2176,11 +2171,11 @@ static void test_thread_description(void)
     ok(!status, "Failed to get thread info.\n");
     ok(len == sizeof(*thread_desc) + desc_len, "Unexpected structure length %u.\n", len);
 
-    ok(thread_desc->Length == (desc_len << 16 | desc_len), "Unexpected description length %#x.\n",
-            thread_desc->Length);
-    ok(thread_desc->Description == (WCHAR *)(thread_desc + 1), "Unexpected description string pointer %p, %p.\n",
-            thread_desc->Description, thread_desc);
-    ok(!memcmp(thread_desc->Description, desc, desc_len), "Unexpected description string.\n");
+    ok(thread_desc->Description.Length == desc_len && thread_desc->Description.MaximumLength == desc_len,
+            "Unexpected description length %u.\n", thread_desc->Description.Length);
+    ok(thread_desc->Description.Buffer == (WCHAR *)(thread_desc + 1),
+            "Unexpected description string pointer %p, %p.\n", thread_desc->Description.Buffer, thread_desc);
+    ok(!memcmp(thread_desc->Description.Buffer, desc, desc_len), "Unexpected description string.\n");
 
     /* Partial results. */
     len = 0;
@@ -2193,7 +2188,7 @@ static void test_thread_description(void)
     ok(len == sizeof(*thread_desc) + desc_len, "Unexpected structure length %u.\n", len);
 
     /* Change description. */
-    thread_desc->Length = 8 << 16 | 8;
+    thread_desc->Description.Length = thread_desc->Description.MaximumLength = 8;
     lstrcpyW((WCHAR *)(thread_desc + 1), L"desc");
 
     status = pNtSetInformationThread(GetCurrentThread(), ThreadDescription, thread_desc, sizeof(*thread_desc));
@@ -2211,7 +2206,7 @@ static void test_thread_description(void)
     status = NtSetInformationThread(GetCurrentThread(), ThreadDescription, NULL, sizeof(*thread_desc));
     ok(status == STATUS_ACCESS_VIOLATION, "Unexpected status %#x.\n", status);
 
-    thread_desc->Description = NULL;
+    thread_desc->Description.Buffer = NULL;
     status = pNtSetInformationThread(GetCurrentThread(), ThreadDescription, thread_desc, sizeof(*thread_desc));
     ok(status == STATUS_ACCESS_VIOLATION, "Unexpected status %#x.\n", status);
 
@@ -2228,8 +2223,7 @@ static void test_thread_description(void)
     hr = pSetThreadDescription(GetCurrentThread(), L"123");
     ok(hr == HRESULT_FROM_NT(STATUS_SUCCESS), "Failed to set thread description, hr %#x.\n", hr);
 
-    thread_desc->Length = 0;
-    thread_desc->Description = NULL;
+    memset(thread_desc, 0, sizeof(*thread_desc));
     status = pNtSetInformationThread(GetCurrentThread(), ThreadDescription, thread_desc, sizeof(*thread_desc));
     ok(!status, "Failed to set thread description, status %#x.\n", status);
 
@@ -2261,11 +2255,6 @@ static void init_funcs(void)
     X(IsWow64Process);
     X(SetThreadErrorMode);
     X(GetThreadErrorMode);
-    X(ActivateActCtx);
-    X(CreateActCtxW);
-    X(DeactivateActCtx);
-    X(GetCurrentActCtx);
-    X(ReleaseActCtx);
 
     X(CreateThreadpool);
     X(CloseThreadpool);
