@@ -519,6 +519,8 @@ static void vmr_destroy(struct strmbase_renderer *iface)
     BaseControlWindow_Destroy(&filter->baseControlWindow);
     strmbase_renderer_cleanup(&filter->renderer);
     CoTaskMemFree(filter);
+
+    InterlockedDecrement(&object_locks);
 }
 
 static HRESULT vmr_query_interface(struct strmbase_renderer *iface, REFIID iid, void **out)
@@ -620,63 +622,71 @@ static HRESULT WINAPI VMR9_GetSourceRect(BaseControlVideo* This, RECT *pSourceRe
     return S_OK;
 }
 
-static HRESULT WINAPI VMR9_GetStaticImage(BaseControlVideo* This, LONG *pBufferSize, LONG *pDIBImage)
+static HRESULT WINAPI VMR9_GetStaticImage(BaseControlVideo *iface, LONG *size, LONG *image)
 {
-    struct quartz_vmr* pVMR9 = impl_from_BaseControlVideo(This);
-    AM_MEDIA_TYPE *amt = &pVMR9->renderer.sink.pin.mt;
-    BITMAPINFOHEADER *bmiHeader;
-    LONG needed_size;
-    char *ptr;
+    struct quartz_vmr *filter = impl_from_BaseControlVideo(iface);
+    const AM_MEDIA_TYPE *mt = &filter->renderer.sink.pin.mt;
+    IDirect3DSurface9 *rt = NULL, *surface = NULL;
+    D3DLOCKED_RECT locked_rect;
+    IDirect3DDevice9 *device;
+    unsigned int row_size;
+    BITMAPINFOHEADER bih;
+    LONG i, size_left;
+    char *dst;
+    HRESULT hr;
 
-    FIXME("(%p/%p)->(%p, %p): partial stub\n", pVMR9, This, pBufferSize, pDIBImage);
+    TRACE("filter %p, size %d, image %p.\n", filter, *size, image);
 
-    EnterCriticalSection(&pVMR9->renderer.filter.csFilter);
+    EnterCriticalSection(&filter->renderer.csRenderLock);
+    device = filter->allocator_d3d9_dev;
 
-    if (!pVMR9->renderer.pMediaSample)
+    if (IsEqualGUID(&mt->formattype, &FORMAT_VideoInfo))
+        bih = ((VIDEOINFOHEADER *)mt->pbFormat)->bmiHeader;
+    else if (IsEqualGUID(&mt->formattype, &FORMAT_VideoInfo2))
+        bih = ((VIDEOINFOHEADER2 *)mt->pbFormat)->bmiHeader;
+    bih.biSizeImage = bih.biWidth * bih.biHeight * bih.biBitCount / 8;
+
+    if (!image)
     {
-         LeaveCriticalSection(&pVMR9->renderer.filter.csFilter);
-         return (pVMR9->renderer.filter.state == State_Paused ? E_UNEXPECTED : VFW_E_NOT_PAUSED);
-    }
-
-    if (IsEqualIID(&amt->formattype, &FORMAT_VideoInfo))
-    {
-        bmiHeader = &((VIDEOINFOHEADER *)amt->pbFormat)->bmiHeader;
-    }
-    else if (IsEqualIID(&amt->formattype, &FORMAT_VideoInfo2))
-    {
-        bmiHeader = &((VIDEOINFOHEADER2 *)amt->pbFormat)->bmiHeader;
-    }
-    else
-    {
-        FIXME("Unknown type %s\n", debugstr_guid(&amt->subtype));
-        LeaveCriticalSection(&pVMR9->renderer.filter.csFilter);
-        return VFW_E_RUNTIME_ERROR;
-    }
-
-    needed_size = bmiHeader->biSize;
-    needed_size += IMediaSample_GetActualDataLength(pVMR9->renderer.pMediaSample);
-
-    if (!pDIBImage)
-    {
-        *pBufferSize = needed_size;
-        LeaveCriticalSection(&pVMR9->renderer.filter.csFilter);
+        *size = sizeof(BITMAPINFOHEADER) + bih.biSizeImage;
+        LeaveCriticalSection(&filter->renderer.csRenderLock);
         return S_OK;
     }
 
-    if (needed_size < *pBufferSize)
+    if (FAILED(hr = IDirect3DDevice9_GetRenderTarget(device, 0, &rt)))
+        goto out;
+
+    if (FAILED(hr = IDirect3DDevice9_CreateOffscreenPlainSurface(device, bih.biWidth,
+            bih.biHeight, D3DFMT_X8R8G8B8, D3DPOOL_SYSTEMMEM, &surface, NULL)))
+        goto out;
+
+    if (FAILED(hr = IDirect3DDevice9_GetRenderTargetData(device, rt, surface)))
+        goto out;
+
+    if (FAILED(hr = IDirect3DSurface9_LockRect(surface, &locked_rect, NULL, D3DLOCK_READONLY)))
+        goto out;
+
+    size_left = *size;
+    memcpy(image, &bih, min(size_left, sizeof(BITMAPINFOHEADER)));
+    size_left -= sizeof(BITMAPINFOHEADER);
+
+    dst = (char *)image + sizeof(BITMAPINFOHEADER);
+    row_size = bih.biWidth * bih.biBitCount / 8;
+
+    for (i = 0; i < bih.biHeight && size_left > 0; ++i)
     {
-        ERR("Buffer too small %u/%u\n", needed_size, *pBufferSize);
-        LeaveCriticalSection(&pVMR9->renderer.filter.csFilter);
-        return E_FAIL;
+        memcpy(dst, (char *)locked_rect.pBits + (i * locked_rect.Pitch), min(row_size, size_left));
+        dst += row_size;
+        size_left -= row_size;
     }
-    *pBufferSize = needed_size;
 
-    memcpy(pDIBImage, bmiHeader, bmiHeader->biSize);
-    IMediaSample_GetPointer(pVMR9->renderer.pMediaSample, (BYTE **)&ptr);
-    memcpy((char *)pDIBImage + bmiHeader->biSize, ptr, IMediaSample_GetActualDataLength(pVMR9->renderer.pMediaSample));
+    IDirect3DSurface9_UnlockRect(surface);
 
-    LeaveCriticalSection(&pVMR9->renderer.filter.csFilter);
-    return S_OK;
+out:
+    if (surface) IDirect3DSurface9_Release(surface);
+    if (rt) IDirect3DSurface9_Release(rt);
+    LeaveCriticalSection(&filter->renderer.csRenderLock);
+    return hr;
 }
 
 static HRESULT WINAPI VMR9_GetTargetRect(BaseControlVideo* This, RECT *pTargetRect)
@@ -2200,7 +2210,7 @@ static const IOverlayVtbl overlay_vtbl =
     overlay_Unadvise,
 };
 
-static HRESULT vmr_create(IUnknown *outer, void **out, const CLSID *clsid)
+static HRESULT vmr_create(IUnknown *outer, IUnknown **out, const CLSID *clsid)
 {
     HRESULT hr;
     struct quartz_vmr* pVMR;
@@ -2241,7 +2251,7 @@ static HRESULT vmr_create(IUnknown *outer, void **out, const CLSID *clsid)
     if (FAILED(hr))
         goto fail;
 
-    hr = strmbase_window_init(&pVMR->baseControlWindow, &IVideoWindow_VTable,
+    hr = video_window_init(&pVMR->baseControlWindow, &IVideoWindow_VTable,
             &pVMR->renderer.filter, &pVMR->renderer.sink.pin, &renderer_BaseWindowFuncTable);
     if (FAILED(hr))
         goto fail;
@@ -2249,7 +2259,7 @@ static HRESULT vmr_create(IUnknown *outer, void **out, const CLSID *clsid)
     if (FAILED(hr = BaseWindowImpl_PrepareWindow(&pVMR->baseControlWindow.baseWindow)))
         goto fail;
 
-    hr = strmbase_video_init(&pVMR->baseControlVideo, &pVMR->renderer.filter,
+    hr = basic_video_init(&pVMR->baseControlVideo, &pVMR->renderer.filter,
             &pVMR->renderer.sink.pin, &renderer_BaseControlVideoFuncTable);
     if (FAILED(hr))
         goto fail;
@@ -2270,14 +2280,14 @@ fail:
     return hr;
 }
 
-HRESULT VMR7Impl_create(IUnknown *outer_unk, LPVOID *ppv)
+HRESULT vmr7_create(IUnknown *outer, IUnknown **out)
 {
-    return vmr_create(outer_unk, ppv, &CLSID_VideoMixingRenderer);
+    return vmr_create(outer, out, &CLSID_VideoMixingRenderer);
 }
 
-HRESULT VMR9Impl_create(IUnknown *outer_unk, LPVOID *ppv)
+HRESULT vmr9_create(IUnknown *outer, IUnknown **out)
 {
-    return vmr_create(outer_unk, ppv, &CLSID_VideoMixingRenderer9);
+    return vmr_create(outer, out, &CLSID_VideoMixingRenderer9);
 }
 
 
