@@ -215,7 +215,7 @@ void *alloc_object( const struct object_ops *ops )
 }
 
 /* free an object once it has been destroyed */
-void free_object( struct object *obj )
+static void free_object( struct object *obj )
 {
     free( obj->sd );
 #ifdef DEBUG_OBJECTS
@@ -425,6 +425,7 @@ void release_object( void *ptr )
         assert( !obj->handle_count );
         /* if the refcount is 0, nobody can be in the wait queue */
         assert( list_empty( &obj->wait_queue ));
+        free_kernel_objects( obj );
         unlink_named_object( obj );
         obj->ops->destroy( obj );
         free_object( obj );
